@@ -117,7 +117,6 @@ void Connection::Final()
 
 Connection::Connection(const char* host, int port, bool tls) :
 	m_host(host), m_port(port), m_tls(tls)
-
 {
 	debug("Creating Connection");
 
@@ -214,7 +213,7 @@ bool Connection::Bind()
 	struct addrinfo addr_hints, *addr_list, *addr;
 
 	memset(&addr_hints, 0, sizeof(addr_hints));
-	addr_hints.ai_family = AF_UNSPEC;    // Allow IPv4 or IPv6
+	addr_hints.ai_family = m_ipVersion == ipV4 ? AF_INET : m_ipVersion == ipV6 ? AF_INET6 : AF_UNSPEC;
 	addr_hints.ai_socktype = SOCK_STREAM,
 	addr_hints.ai_flags = AI_PASSIVE;    // For wildcard IP address
 
@@ -374,7 +373,7 @@ char* Connection::ReadLine(char* buffer, int size, int* bytesReadOut)
 			bufAvail = recv(m_socket, m_readBuf, m_readBuf.Size() - 1, 0);
 			if (bufAvail < 0)
 			{
-				ReportError("Could not receive data on socket", nullptr, true);
+				ReportError("Could not receive data on socket from %s", m_host, true);
 				m_broken = true;
 				break;
 			}
@@ -446,7 +445,7 @@ std::unique_ptr<Connection> Connection::Accept()
 	SOCKET socket = accept(m_socket, nullptr, nullptr);
 	if (socket == INVALID_SOCKET && m_status != csCancelled)
 	{
-		ReportError("Could not accept connection", nullptr, true);
+		ReportError("Could not accept connection for %s", m_host, true);
 	}
 	if (socket == INVALID_SOCKET)
 	{
@@ -466,7 +465,7 @@ int Connection::TryRecv(char* buffer, int size)
 
 	if (received < 0)
 	{
-		ReportError("Could not receive data on socket", nullptr, true);
+		ReportError("Could not receive data on socket from %s", m_host, true);
 	}
 
 	return received;
@@ -498,7 +497,7 @@ bool Connection::Recv(char * buffer, int size)
 		// Did the recv succeed?
 		if (received <= 0)
 		{
-			ReportError("Could not receive data on socket", nullptr, true);
+			ReportError("Could not receive data on socket from %s", m_host, true);
 			return false;
 		}
 		bufPtr += received;
@@ -518,7 +517,7 @@ bool Connection::DoConnect()
 	struct addrinfo addr_hints, *addr_list, *addr;
 
 	memset(&addr_hints, 0, sizeof(addr_hints));
-	addr_hints.ai_family = AF_UNSPEC;    /* Allow IPv4 or IPv6 */
+	addr_hints.ai_family = m_ipVersion == ipV4 ? AF_INET : m_ipVersion == ipV6 ? AF_INET6 : AF_UNSPEC;
 	addr_hints.ai_socktype = SOCK_STREAM;
 
 	BString<100> portStr("%d", m_port);
@@ -813,7 +812,7 @@ void Connection::Cancel()
 		int r = shutdown(m_socket, SHUT_RDWR);
 		if (r == -1)
 		{
-			ReportError("Could not shutdown connection", nullptr, true);
+			ReportError("Could not shutdown connection for %s", m_host, true);
 		}
 	}
 }
