@@ -1,7 +1,7 @@
 /*
- *  This file is part of nzbget
+ *  This file is part of nzbget. See <http://nzbget.net>.
  *
- *  Copyright (C) 2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2015-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,12 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *
- * $Revision$
- * $Date$
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -30,20 +25,17 @@
 
 class DiskService : public Service
 {
-private:
-	int					m_iInterval;
-	bool				m_bWaitingRequiredDir;
-	bool				m_bWaitingReported;
-
-	void				CheckDiskSpace();
-	void				CheckRequiredDir();
-
 protected:
-	virtual int			ServiceInterval() { return 200; }
-	virtual void		ServiceWork();
+	virtual int ServiceInterval() { return 200; }
+	virtual void ServiceWork();
 
-public:
-						DiskService();
+private:
+	int m_interval = 0;
+	bool m_waitingRequiredDir = true;
+	bool m_waitingReported = false;
+
+	void CheckDiskSpace();
+	void CheckRequiredDir();
 };
 
 #endif
