@@ -2,7 +2,7 @@
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,10 +59,13 @@ static const char* OPTION_RESTRICTEDUSERNAME	= "RestrictedUsername";
 static const char* OPTION_RESTRICTEDPASSWORD	= "RestrictedPassword";
 static const char* OPTION_ADDUSERNAME			= "AddUsername";
 static const char* OPTION_ADDPASSWORD			= "AddPassword";
+static const char* OPTION_FORMAUTH				= "FormAuth";
 static const char* OPTION_SECURECONTROL			= "SecureControl";
 static const char* OPTION_SECUREPORT			= "SecurePort";
 static const char* OPTION_SECURECERT			= "SecureCert";
 static const char* OPTION_SECUREKEY				= "SecureKey";
+static const char* OPTION_CERTSTORE				= "CertStore";
+static const char* OPTION_CERTCHECK				= "CertCheck";
 static const char* OPTION_AUTHORIZEDIP			= "AuthorizedIP";
 static const char* OPTION_ARTICLETIMEOUT		= "ArticleTimeout";
 static const char* OPTION_URLTIMEOUT			= "UrlTimeout";
@@ -72,8 +75,10 @@ static const char* OPTION_RELOADQUEUE			= "ReloadQueue";
 static const char* OPTION_BROKENLOG				= "BrokenLog";
 static const char* OPTION_NZBLOG				= "NzbLog";
 static const char* OPTION_DECODE				= "Decode";
-static const char* OPTION_RETRIES				= "Retries";
-static const char* OPTION_RETRYINTERVAL			= "RetryInterval";
+static const char* OPTION_ARTICLERETRIES		= "ArticleRetries";
+static const char* OPTION_ARTICLEINTERVAL		= "ArticleInterval";
+static const char* OPTION_URLRETRIES			= "UrlRetries";
+static const char* OPTION_URLINTERVAL			= "UrlInterval";
 static const char* OPTION_TERMINATETIMEOUT		= "TerminateTimeout";
 static const char* OPTION_CONTINUEPARTIAL		= "ContinuePartial";
 static const char* OPTION_URLCONNECTIONS		= "UrlConnections";
@@ -87,13 +92,14 @@ static const char* OPTION_PARCHECK				= "ParCheck";
 static const char* OPTION_PARREPAIR				= "ParRepair";
 static const char* OPTION_PARSCAN				= "ParScan";
 static const char* OPTION_PARQUICK				= "ParQuick";
+static const char* OPTION_POSTSTRATEGY			= "PostStrategy";
+static const char* OPTION_FILENAMING			= "FileNaming";
 static const char* OPTION_PARRENAME				= "ParRename";
 static const char* OPTION_PARBUFFER				= "ParBuffer";
 static const char* OPTION_PARTHREADS			= "ParThreads";
+static const char* OPTION_RARRENAME				= "RarRename";
 static const char* OPTION_HEALTHCHECK			= "HealthCheck";
-static const char* OPTION_SCANSCRIPT			= "ScanScript";
-static const char* OPTION_QUEUESCRIPT			= "QueueScript";
-static const char* OPTION_FEEDSCRIPT			= "FeedScript";
+static const char* OPTION_DIRECTRENAME			= "DirectRename";
 static const char* OPTION_UMASK					= "UMask";
 static const char* OPTION_UPDATEINTERVAL		= "UpdateInterval";
 static const char* OPTION_CURSESNZBNAME			= "CursesNzbName";
@@ -105,7 +111,8 @@ static const char* OPTION_WRITEBUFFER			= "WriteBuffer";
 static const char* OPTION_NZBDIRINTERVAL		= "NzbDirInterval";
 static const char* OPTION_NZBDIRFILEAGE			= "NzbDirFileAge";
 static const char* OPTION_DISKSPACE				= "DiskSpace";
-static const char* OPTION_DUMPCORE				= "DumpCore";
+static const char* OPTION_CRASHTRACE			= "CrashTrace";
+static const char* OPTION_CRASHDUMP				= "CrashDump";
 static const char* OPTION_PARPAUSEQUEUE			= "ParPauseQueue";
 static const char* OPTION_SCRIPTPAUSEQUEUE		= "ScriptPauseQueue";
 static const char* OPTION_NZBCLEANUPDISK		= "NzbCleanupDisk";
@@ -113,15 +120,17 @@ static const char* OPTION_PARTIMELIMIT			= "ParTimeLimit";
 static const char* OPTION_KEEPHISTORY			= "KeepHistory";
 static const char* OPTION_ACCURATERATE			= "AccurateRate";
 static const char* OPTION_UNPACK				= "Unpack";
+static const char* OPTION_DIRECTUNPACK			= "DirectUnpack";
 static const char* OPTION_UNPACKCLEANUPDISK		= "UnpackCleanupDisk";
 static const char* OPTION_UNRARCMD				= "UnrarCmd";
 static const char* OPTION_SEVENZIPCMD			= "SevenZipCmd";
 static const char* OPTION_UNPACKPASSFILE		= "UnpackPassFile";
 static const char* OPTION_UNPACKPAUSEQUEUE		= "UnpackPauseQueue";
 static const char* OPTION_SCRIPTORDER			= "ScriptOrder";
-static const char* OPTION_POSTSCRIPT			= "PostScript";
+static const char* OPTION_EXTENSIONS			= "Extensions";
 static const char* OPTION_EXTCLEANUPDISK		= "ExtCleanupDisk";
 static const char* OPTION_PARIGNOREEXT			= "ParIgnoreExt";
+static const char* OPTION_UNPACKIGNOREEXT		= "UnpackIgnoreExt";
 static const char* OPTION_FEEDHISTORY			= "FeedHistory";
 static const char* OPTION_URLFORCE				= "UrlForce";
 static const char* OPTION_TIMECORRECTION		= "TimeCorrection";
@@ -132,6 +141,7 @@ static const char* OPTION_SHELLOVERRIDE			= "ShellOverride";
 static const char* OPTION_MONTHLYQUOTA			= "MonthlyQuota";
 static const char* OPTION_QUOTASTARTDAY			= "QuotaStartDay";
 static const char* OPTION_DAILYQUOTA			= "DailyQuota";
+static const char* OPTION_REORDERFILES			= "ReorderFiles";
 
 // obsolete options
 static const char* OPTION_POSTLOGKIND			= "PostLogKind";
@@ -155,6 +165,9 @@ static const char* OPTION_RESETLOG				= "ResetLog";
 static const char* OPTION_PARCLEANUPQUEUE		= "ParCleanupQueue";
 static const char* OPTION_DELETECLEANUPDISK		= "DeleteCleanupDisk";
 static const char* OPTION_HISTORYCLEANUPDISK	= "HistoryCleanupDisk";
+static const char* OPTION_SCANSCRIPT			= "ScanScript";
+static const char* OPTION_QUEUESCRIPT			= "QueueScript";
+static const char* OPTION_FEEDSCRIPT			= "FeedScript";
 
 const char* BoolNames[] = { "yes", "no", "true", "false", "1", "0", "on", "off", "enable", "disable", "enabled", "disabled" };
 const int BoolValues[] = { 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0 };
@@ -188,10 +201,13 @@ bool Options::OptEntry::Restricted()
 
 	bool restricted = !strcasecmp(m_name, OPTION_CONTROLIP) ||
 		!strcasecmp(m_name, OPTION_CONTROLPORT) ||
+		!strcasecmp(m_name, OPTION_FORMAUTH) ||
 		!strcasecmp(m_name, OPTION_SECURECONTROL) ||
 		!strcasecmp(m_name, OPTION_SECUREPORT) ||
 		!strcasecmp(m_name, OPTION_SECURECERT) ||
 		!strcasecmp(m_name, OPTION_SECUREKEY) ||
+		!strcasecmp(m_name, OPTION_CERTSTORE) ||
+		!strcasecmp(m_name, OPTION_CERTCHECK) ||
 		!strcasecmp(m_name, OPTION_AUTHORIZEDIP) ||
 		!strcasecmp(m_name, OPTION_DAEMONUSERNAME) ||
 		!strcasecmp(m_name, OPTION_UMASK) ||
@@ -325,6 +341,7 @@ void Options::Init(const char* exeName, const char* configFilename, bool noConfi
 		return;
 	}
 
+	ConvertOldOptions(&m_optEntries);
 	InitOptions();
 	CheckOptions();
 
@@ -416,10 +433,13 @@ void Options::InitDefaults()
 	SetOption(OPTION_ADDUSERNAME, "");
 	SetOption(OPTION_ADDPASSWORD, "");
 	SetOption(OPTION_CONTROLPORT, "6789");
+	SetOption(OPTION_FORMAUTH, "no");
 	SetOption(OPTION_SECURECONTROL, "no");
 	SetOption(OPTION_SECUREPORT, "6791");
 	SetOption(OPTION_SECURECERT, "");
 	SetOption(OPTION_SECUREKEY, "");
+	SetOption(OPTION_CERTSTORE, "");
+	SetOption(OPTION_CERTCHECK, "no");
 	SetOption(OPTION_AUTHORIZEDIP, "");
 	SetOption(OPTION_ARTICLETIMEOUT, "60");
 	SetOption(OPTION_URLTIMEOUT, "60");
@@ -429,8 +449,10 @@ void Options::InitDefaults()
 	SetOption(OPTION_BROKENLOG, "yes");
 	SetOption(OPTION_NZBLOG, "yes");
 	SetOption(OPTION_DECODE, "yes");
-	SetOption(OPTION_RETRIES, "3");
-	SetOption(OPTION_RETRYINTERVAL, "10");
+	SetOption(OPTION_ARTICLERETRIES, "3");
+	SetOption(OPTION_ARTICLEINTERVAL, "10");
+	SetOption(OPTION_URLRETRIES, "3");
+	SetOption(OPTION_URLINTERVAL, "10");
 	SetOption(OPTION_TERMINATETIMEOUT, "600");
 	SetOption(OPTION_CONTINUEPARTIAL, "no");
 	SetOption(OPTION_URLCONNECTIONS, "4");
@@ -444,15 +466,16 @@ void Options::InitDefaults()
 	SetOption(OPTION_PARREPAIR, "yes");
 	SetOption(OPTION_PARSCAN, "extended");
 	SetOption(OPTION_PARQUICK, "yes");
+	SetOption(OPTION_POSTSTRATEGY, "sequential");
+	SetOption(OPTION_FILENAMING, "article");
 	SetOption(OPTION_PARRENAME, "yes");
 	SetOption(OPTION_PARBUFFER, "16");
 	SetOption(OPTION_PARTHREADS, "1");
+	SetOption(OPTION_RARRENAME, "yes");
 	SetOption(OPTION_HEALTHCHECK, "none");
+	SetOption(OPTION_DIRECTRENAME, "no");
 	SetOption(OPTION_SCRIPTORDER, "");
-	SetOption(OPTION_POSTSCRIPT, "");
-	SetOption(OPTION_SCANSCRIPT, "");
-	SetOption(OPTION_QUEUESCRIPT, "");
-	SetOption(OPTION_FEEDSCRIPT, "");
+	SetOption(OPTION_EXTENSIONS, "");
 	SetOption(OPTION_DAEMONUSERNAME, "root");
 	SetOption(OPTION_UMASK, "1000");
 	SetOption(OPTION_UPDATEINTERVAL, "200");
@@ -465,7 +488,8 @@ void Options::InitDefaults()
 	SetOption(OPTION_NZBDIRINTERVAL, "5");
 	SetOption(OPTION_NZBDIRFILEAGE, "60");
 	SetOption(OPTION_DISKSPACE, "250");
-	SetOption(OPTION_DUMPCORE, "no");
+	SetOption(OPTION_CRASHTRACE, "no");
+	SetOption(OPTION_CRASHDUMP, "no");
 	SetOption(OPTION_PARPAUSEQUEUE, "no");
 	SetOption(OPTION_SCRIPTPAUSEQUEUE, "no");
 	SetOption(OPTION_NZBCLEANUPDISK, "no");
@@ -473,6 +497,7 @@ void Options::InitDefaults()
 	SetOption(OPTION_KEEPHISTORY, "7");
 	SetOption(OPTION_ACCURATERATE, "no");
 	SetOption(OPTION_UNPACK, "no");
+	SetOption(OPTION_DIRECTUNPACK, "no");
 	SetOption(OPTION_UNPACKCLEANUPDISK, "no");
 #ifdef WIN32
 	SetOption(OPTION_UNRARCMD, "unrar.exe");
@@ -485,6 +510,7 @@ void Options::InitDefaults()
 	SetOption(OPTION_UNPACKPAUSEQUEUE, "no");
 	SetOption(OPTION_EXTCLEANUPDISK, "");
 	SetOption(OPTION_PARIGNOREEXT, "");
+	SetOption(OPTION_UNPACKIGNOREEXT, "");
 	SetOption(OPTION_FEEDHISTORY, "7");
 	SetOption(OPTION_URLFORCE, "yes");
 	SetOption(OPTION_TIMECORRECTION, "0");
@@ -495,6 +521,7 @@ void Options::InitDefaults()
 	SetOption(OPTION_MONTHLYQUOTA, "0");
 	SetOption(OPTION_QUOTASTARTDAY, "1");
 	SetOption(OPTION_DAILYQUOTA, "0");
+	SetOption(OPTION_REORDERFILES, "no");
 }
 
 void Options::InitOptFile()
@@ -641,10 +668,7 @@ void Options::InitOptions()
 
 	m_configTemplate		= GetOption(OPTION_CONFIGTEMPLATE);
 	m_scriptOrder			= GetOption(OPTION_SCRIPTORDER);
-	m_postScript			= GetOption(OPTION_POSTSCRIPT);
-	m_scanScript			= GetOption(OPTION_SCANSCRIPT);
-	m_queueScript			= GetOption(OPTION_QUEUESCRIPT);
-	m_feedScript			= GetOption(OPTION_FEEDSCRIPT);
+	m_extensions			= GetOption(OPTION_EXTENSIONS);
 	m_controlIp				= GetOption(OPTION_CONTROLIP);
 	m_controlUsername		= GetOption(OPTION_CONTROLUSERNAME);
 	m_controlPassword		= GetOption(OPTION_CONTROLPASSWORD);
@@ -654,6 +678,7 @@ void Options::InitOptions()
 	m_addPassword			= GetOption(OPTION_ADDPASSWORD);
 	m_secureCert			= GetOption(OPTION_SECURECERT);
 	m_secureKey				= GetOption(OPTION_SECUREKEY);
+	m_certStore				= GetOption(OPTION_CERTSTORE);
 	m_authorizedIp			= GetOption(OPTION_AUTHORIZEDIP);
 	m_lockFile				= GetOption(OPTION_LOCKFILE);
 	m_daemonUsername		= GetOption(OPTION_DAEMONUSERNAME);
@@ -663,14 +688,17 @@ void Options::InitOptions()
 	m_unpackPassFile		= GetOption(OPTION_UNPACKPASSFILE);
 	m_extCleanupDisk		= GetOption(OPTION_EXTCLEANUPDISK);
 	m_parIgnoreExt			= GetOption(OPTION_PARIGNOREEXT);
+	m_unpackIgnoreExt		= GetOption(OPTION_UNPACKIGNOREEXT);
 	m_shellOverride			= GetOption(OPTION_SHELLOVERRIDE);
 
 	m_downloadRate			= ParseIntValue(OPTION_DOWNLOADRATE, 10) * 1024;
 	m_articleTimeout		= ParseIntValue(OPTION_ARTICLETIMEOUT, 10);
 	m_urlTimeout			= ParseIntValue(OPTION_URLTIMEOUT, 10);
 	m_terminateTimeout		= ParseIntValue(OPTION_TERMINATETIMEOUT, 10);
-	m_retries				= ParseIntValue(OPTION_RETRIES, 10);
-	m_retryInterval			= ParseIntValue(OPTION_RETRYINTERVAL, 10);
+	m_articleRetries		= ParseIntValue(OPTION_ARTICLERETRIES, 10);
+	m_articleInterval		= ParseIntValue(OPTION_ARTICLEINTERVAL, 10);
+	m_urlRetries			= ParseIntValue(OPTION_URLRETRIES, 10);
+	m_urlInterval			= ParseIntValue(OPTION_URLINTERVAL, 10);
 	m_controlPort			= ParseIntValue(OPTION_CONTROLPORT, 10);
 	m_securePort			= ParseIntValue(OPTION_SECUREPORT, 10);
 	m_urlConnections		= ParseIntValue(OPTION_URLCONNECTIONS, 10);
@@ -710,6 +738,8 @@ void Options::InitOptions()
 	m_parRepair				= (bool)ParseEnumValue(OPTION_PARREPAIR, BoolCount, BoolNames, BoolValues);
 	m_parQuick				= (bool)ParseEnumValue(OPTION_PARQUICK, BoolCount, BoolNames, BoolValues);
 	m_parRename				= (bool)ParseEnumValue(OPTION_PARRENAME, BoolCount, BoolNames, BoolValues);
+	m_rarRename				= (bool)ParseEnumValue(OPTION_RARRENAME, BoolCount, BoolNames, BoolValues);
+	m_directRename			= (bool)ParseEnumValue(OPTION_DIRECTRENAME, BoolCount, BoolNames, BoolValues);
 	m_reloadQueue			= (bool)ParseEnumValue(OPTION_RELOADQUEUE, BoolCount, BoolNames, BoolValues);
 	m_cursesNzbName			= (bool)ParseEnumValue(OPTION_CURSESNZBNAME, BoolCount, BoolNames, BoolValues);
 	m_cursesTime			= (bool)ParseEnumValue(OPTION_CURSESTIME, BoolCount, BoolNames, BoolValues);
@@ -717,16 +747,21 @@ void Options::InitOptions()
 	m_crcCheck				= (bool)ParseEnumValue(OPTION_CRCCHECK, BoolCount, BoolNames, BoolValues);
 	m_directWrite			= (bool)ParseEnumValue(OPTION_DIRECTWRITE, BoolCount, BoolNames, BoolValues);
 	m_decode				= (bool)ParseEnumValue(OPTION_DECODE, BoolCount, BoolNames, BoolValues);
-	m_dumpCore				= (bool)ParseEnumValue(OPTION_DUMPCORE, BoolCount, BoolNames, BoolValues);
+	m_crashTrace			= (bool)ParseEnumValue(OPTION_CRASHTRACE, BoolCount, BoolNames, BoolValues);
+	m_crashDump				= (bool)ParseEnumValue(OPTION_CRASHDUMP, BoolCount, BoolNames, BoolValues);
 	m_parPauseQueue			= (bool)ParseEnumValue(OPTION_PARPAUSEQUEUE, BoolCount, BoolNames, BoolValues);
 	m_scriptPauseQueue		= (bool)ParseEnumValue(OPTION_SCRIPTPAUSEQUEUE, BoolCount, BoolNames, BoolValues);
 	m_nzbCleanupDisk		= (bool)ParseEnumValue(OPTION_NZBCLEANUPDISK, BoolCount, BoolNames, BoolValues);
 	m_accurateRate			= (bool)ParseEnumValue(OPTION_ACCURATERATE, BoolCount, BoolNames, BoolValues);
+	m_formAuth				= (bool)ParseEnumValue(OPTION_FORMAUTH, BoolCount, BoolNames, BoolValues);
 	m_secureControl			= (bool)ParseEnumValue(OPTION_SECURECONTROL, BoolCount, BoolNames, BoolValues);
 	m_unpack				= (bool)ParseEnumValue(OPTION_UNPACK, BoolCount, BoolNames, BoolValues);
+	m_directUnpack			= (bool)ParseEnumValue(OPTION_DIRECTUNPACK, BoolCount, BoolNames, BoolValues);
 	m_unpackCleanupDisk		= (bool)ParseEnumValue(OPTION_UNPACKCLEANUPDISK, BoolCount, BoolNames, BoolValues);
 	m_unpackPauseQueue		= (bool)ParseEnumValue(OPTION_UNPACKPAUSEQUEUE, BoolCount, BoolNames, BoolValues);
 	m_urlForce				= (bool)ParseEnumValue(OPTION_URLFORCE, BoolCount, BoolNames, BoolValues);
+	m_certCheck				= (bool)ParseEnumValue(OPTION_CERTCHECK, BoolCount, BoolNames, BoolValues);
+	m_reorderFiles			= (bool)ParseEnumValue(OPTION_REORDERFILES, BoolCount, BoolNames, BoolValues);
 
 	const char* OutputModeNames[] = { "loggable", "logable", "log", "colored", "color", "ncurses", "curses" };
 	const int OutputModeValues[] = { omLoggable, omLoggable, omLoggable, omColored, omColored, omNCurses, omNCurses };
@@ -743,6 +778,16 @@ void Options::InitOptions()
 	const int ParScanCount = 4;
 	m_parScan = (EParScan)ParseEnumValue(OPTION_PARSCAN, ParScanCount, ParScanNames, ParScanValues);
 
+	const char* PostStrategyNames[] = { "sequential", "balanced", "aggressive", "rocket" };
+	const int PostStrategyValues[] = { ppSequential, ppBalanced, ppAggressive, ppRocket };
+	const int PostStrategyCount = 4;
+	m_postStrategy = (EPostStrategy)ParseEnumValue(OPTION_POSTSTRATEGY, PostStrategyCount, PostStrategyNames, PostStrategyValues);
+
+	const char* FileNamingNames[] = { "auto", "article", "nzb" };
+	const int FileNamingValues[] = { nfAuto, nfArticle, nfNzb };
+	const int FileNamingCount = 4;
+	m_fileNaming = (EFileNaming)ParseEnumValue(OPTION_FILENAMING, FileNamingCount, FileNamingNames, FileNamingValues);
+
 	const char* HealthCheckNames[] = { "pause", "delete", "park", "none" };
 	const int HealthCheckValues[] = { hcPause, hcDelete, hcPark, hcNone };
 	const int HealthCheckCount = 4;
@@ -965,6 +1010,16 @@ void Options::InitServers()
 			m_tls |= tls;
 		}
 
+		const char* nipversion = GetOption(BString<100>("Server%i.IpVersion", n));
+		int ipversion = 0;
+		if (nipversion)
+		{
+			const char* IpVersionNames[] = {"auto", "ipv4", "ipv6"};
+			const int IpVersionValues[] = {0, 4, 6};
+			const int IpVersionCount = 3;
+			ipversion = ParseEnumValue(BString<100>("Server%i.IpVersion", n), IpVersionCount, IpVersionNames, IpVersionValues);
+		}
+
 		const char* ncipher = GetOption(BString<100>("Server%i.Cipher", n));
 		const char* nconnections = GetOption(BString<100>("Server%i.Connections", n));
 		const char* nretention = GetOption(BString<100>("Server%i.Retention", n));
@@ -985,6 +1040,7 @@ void Options::InitServers()
 				m_extender->AddNewsServer(n, active, nname,
 					nhost,
 					nport ? atoi(nport) : 119,
+					ipversion,
 					nusername, npassword,
 					joinGroup, tls, ncipher,
 					nconnections ? atoi(nconnections) : 1,
@@ -1018,10 +1074,10 @@ void Options::InitCategories()
 			unpack = (bool)ParseEnumValue(BString<100>("Category%i.Unpack", n), BoolCount, BoolNames, BoolValues);
 		}
 
-		const char* npostscript = GetOption(BString<100>("Category%i.PostScript", n));
+		const char* nextensions = GetOption(BString<100>("Category%i.Extensions", n));
 		const char* naliases = GetOption(BString<100>("Category%i.Aliases", n));
 
-		bool definition = nname || ndestdir || nunpack || npostscript || naliases;
+		bool definition = nname || ndestdir || nunpack || nextensions || naliases;
 		bool completed = nname && strlen(nname) > 0;
 
 		if (!definition)
@@ -1037,7 +1093,7 @@ void Options::InitCategories()
 				CheckDir(destDir, BString<100>("Category%i.DestDir", n), m_destDir, false, false);
 			}
 
-			m_categories.emplace_back(nname, destDir, unpack, npostscript);
+			m_categories.emplace_back(nname, destDir, unpack, nextensions);
 			Category& category = m_categories.back();
 
 			// split Aliases into tokens and create items for each token
@@ -1068,7 +1124,7 @@ void Options::InitFeeds()
 		const char* nurl = GetOption(BString<100>("Feed%i.URL", n));
 		const char* nfilter = GetOption(BString<100>("Feed%i.Filter", n));
 		const char* ncategory = GetOption(BString<100>("Feed%i.Category", n));
-		const char* nfeedscript = GetOption(BString<100>("Feed%i.FeedScript", n));
+		const char* nextensions = GetOption(BString<100>("Feed%i.Extensions", n));
 
 		const char* nbacklog = GetOption(BString<100>("Feed%i.Backlog", n));
 		bool backlog = true;
@@ -1088,7 +1144,7 @@ void Options::InitFeeds()
 		const char* npriority = GetOption(BString<100>("Feed%i.Priority", n));
 
 		bool definition = nname || nurl || nfilter || ncategory || nbacklog || npausenzb ||
-			ninterval || npriority || nfeedscript;
+			ninterval || npriority || nextensions;
 		bool completed = nurl;
 
 		if (!definition)
@@ -1101,7 +1157,7 @@ void Options::InitFeeds()
 			if (m_extender)
 			{
 				m_extender->AddFeed(n, nname, nurl, ninterval ? atoi(ninterval) : 0, nfilter,
-					backlog, pauseNzb, ncategory, npriority ? atoi(npriority) : 0, nfeedscript);
+					backlog, pauseNzb, ncategory, npriority ? atoi(npriority) : 0, nextensions);
 			}
 		}
 		else
@@ -1170,13 +1226,6 @@ void Options::InitScheduler()
 			continue;
 		}
 
-		int weekDaysVal = 0;
-		if (weekDays && !ParseWeekDays(weekDays, &weekDaysVal))
-		{
-			ConfigError("Invalid value for option \"Task%i.WeekDays\": \"%s\"", n, weekDays);
-			continue;
-		}
-
 		if (taskCommand == scDownloadRate)
 		{
 			if (param)
@@ -1207,36 +1256,60 @@ void Options::InitScheduler()
 			continue;
 		}
 
-		int hours, minutes;
-		Tokenizer tok(time, ";,");
-		while (const char* oneTime = tok.Next())
-		{
-			if (!ParseTime(oneTime, &hours, &minutes))
-			{
-				ConfigError("Invalid value for option \"Task%i.Time\": \"%s\"", n, oneTime);
-				break;
-			}
+		CreateSchedulerTask(n, time, weekDays, taskCommand, param);
+	}
+}
 
-			if (m_extender)
+void Options::CreateSchedulerTask(int id, const char* time, const char* weekDays,
+	ESchedulerCommand command, const char* param)
+{
+	if (!id)
+	{
+		m_configLine = 0;
+	}
+
+	int weekDaysVal = 0;
+	if (weekDays && !ParseWeekDays(weekDays, &weekDaysVal))
+	{
+		ConfigError("Invalid value for option \"Task%i.WeekDays\": \"%s\"", id, weekDays);
+		return;
+	}
+
+	int hours, minutes;
+	Tokenizer tok(time, ";,");
+	while (const char* oneTime = tok.Next())
+	{
+		if (!ParseTime(oneTime, &hours, &minutes))
+		{
+			ConfigError("Invalid value for option \"Task%i.Time\": \"%s\"", id, oneTime);
+			return;
+		}
+
+		if (m_extender)
+		{
+			if (hours == -2)
 			{
-				if (hours == -1)
+				for (int everyHour = 0; everyHour < 24; everyHour++)
 				{
-					for (int everyHour = 0; everyHour < 24; everyHour++)
-					{
-						m_extender->AddTask(n, everyHour, minutes, weekDaysVal, taskCommand, param);
-					}
-				}
-				else
-				{
-					m_extender->AddTask(n, hours, minutes, weekDaysVal, taskCommand, param);
+					m_extender->AddTask(id, everyHour, minutes, weekDaysVal, command, param);
 				}
 			}
+			else
+			{
+				m_extender->AddTask(id, hours, minutes, weekDaysVal, command, param);
+			}
 		}
 	}
 }
 
 bool Options::ParseTime(const char* time, int* hours, int* minutes)
 {
+	if (!strcmp(time, "*"))
+	{
+		*hours = -1;
+		return true;
+	}
+
 	int colons = 0;
 	const char* p = time;
 	while (*p)
@@ -1265,7 +1338,7 @@ bool Options::ParseTime(const char* time, int* hours, int* minutes)
 
 	if (time[0] == '*')
 	{
-		*hours = -1;
+		*hours = -2;
 	}
 	else
 	{
@@ -1464,7 +1537,8 @@ bool Options::ValidateOptionName(const char* optname, const char* optvalue)
 			!strcasecmp(p, ".password") || !strcasecmp(p, ".joingroup") ||
 			!strcasecmp(p, ".encryption") || !strcasecmp(p, ".connections") ||
 			!strcasecmp(p, ".cipher") || !strcasecmp(p, ".group") ||
-			!strcasecmp(p, ".retention") || !strcasecmp(p, ".optional")))
+			!strcasecmp(p, ".retention") || !strcasecmp(p, ".optional") ||
+			!strcasecmp(p, ".notes") || !strcasecmp(p, ".ipversion")))
 		{
 			return true;
 		}
@@ -1486,7 +1560,7 @@ bool Options::ValidateOptionName(const char* optname, const char* optvalue)
 	{
 		char* p = (char*)optname + 8;
 		while (*p >= '0' && *p <= '9') p++;
-		if (p && (!strcasecmp(p, ".name") || !strcasecmp(p, ".destdir") || !strcasecmp(p, ".postscript") ||
+		if (p && (!strcasecmp(p, ".name") || !strcasecmp(p, ".destdir") || !strcasecmp(p, ".extensions") ||
 			!strcasecmp(p, ".unpack") || !strcasecmp(p, ".aliases")))
 		{
 			return true;
@@ -1499,7 +1573,7 @@ bool Options::ValidateOptionName(const char* optname, const char* optvalue)
 		while (*p >= '0' && *p <= '9') p++;
 		if (p && (!strcasecmp(p, ".name") || !strcasecmp(p, ".url") || !strcasecmp(p, ".interval") ||
 			 !strcasecmp(p, ".filter") || !strcasecmp(p, ".backlog") || !strcasecmp(p, ".pausenzb") ||
-			 !strcasecmp(p, ".category") || !strcasecmp(p, ".priority") || !strcasecmp(p, ".feedscript")))
+			 !strcasecmp(p, ".category") || !strcasecmp(p, ".priority") || !strcasecmp(p, ".extensions")))
 		{
 			return true;
 		}
@@ -1532,21 +1606,27 @@ bool Options::ValidateOptionName(const char* optname, const char* optvalue)
 		ConfigWarn("Option \"%s\" is obsolete, ignored", optname);
 		return true;
 	}
+
 	if (!strcasecmp(optname, OPTION_POSTPROCESS) ||
 		!strcasecmp(optname, OPTION_NZBPROCESS) ||
 		!strcasecmp(optname, OPTION_NZBADDEDPROCESS))
 	{
 		if (optvalue && strlen(optvalue) > 0)
 		{
-			ConfigError("Option \"%s\" is obsolete, ignored, use \"%s\" and \"%s\" instead", optname, OPTION_SCRIPTDIR,
-				!strcasecmp(optname, OPTION_POSTPROCESS) ? OPTION_POSTSCRIPT :
-				!strcasecmp(optname, OPTION_NZBPROCESS) ? OPTION_SCANSCRIPT :
-				!strcasecmp(optname, OPTION_NZBADDEDPROCESS) ? OPTION_QUEUESCRIPT :
-				"ERROR");
+			ConfigError("Option \"%s\" is obsolete, ignored, use \"%s\" and \"%s\" instead",
+				optname, OPTION_SCRIPTDIR, OPTION_EXTENSIONS);
 		}
 		return true;
 	}
 
+	if (!strcasecmp(optname, OPTION_SCANSCRIPT) ||
+		!strcasecmp(optname, OPTION_QUEUESCRIPT) ||
+		!strcasecmp(optname, OPTION_FEEDSCRIPT))
+	{
+		// will be automatically converted into "Extensions"
+		return true;
+	}
+
 	if (!strcasecmp(optname, OPTION_CREATELOG) || !strcasecmp(optname, OPTION_RESETLOG))
 	{
 		ConfigWarn("Option \"%s\" is obsolete, ignored, use \"%s\" instead", optname, OPTION_WRITELOG);
@@ -1600,16 +1680,22 @@ void Options::ConvertOldOption(CString& option, CString& value)
 		value = "extended";
 	}
 
-	if (!strcasecmp(option, "DefScript"))
+	if (!strcasecmp(option, "DefScript") || !strcasecmp(option, "PostScript"))
 	{
-		option = "PostScript";
+		option = "Extensions";
 	}
 
 	int nameLen = strlen(option);
-	if (!strncasecmp(option, "Category", 8) && nameLen > 10 &&
-		!strcasecmp(option + nameLen - 10, ".DefScript"))
+	if (!strncasecmp(option, "Category", 8) &&
+		((nameLen > 10 && !strcasecmp(option + nameLen - 10, ".DefScript")) ||
+		 (nameLen > 11 && !strcasecmp(option + nameLen - 11, ".PostScript"))))
 	{
-		option.Replace(".DefScript", ".PostScript");
+		option.Replace(".DefScript", ".Extensions");
+		option.Replace(".PostScript", ".Extensions");
+	}
+	if (!strncasecmp(option, "Feed", 4) && nameLen > 11 && !strcasecmp(option + nameLen - 11, ".FeedScript"))
+	{
+		option.Replace(".FeedScript", ".Extensions");
 	}
 
 	if (!strcasecmp(option, "WriteBufferSize"))
@@ -1625,10 +1711,25 @@ void Options::ConvertOldOption(CString& option, CString& value)
 		option = "ArticleTimeout";
 	}
 
+	if (!strcasecmp(option, "Retries"))
+	{
+		option = "ArticleRetries";
+	}
+
+	if (!strcasecmp(option, "RetryInterval"))
+	{
+		option = "ArticleInterval";
+	}
+
 	if (!strcasecmp(option, "CreateBrokenLog"))
 	{
 		option = "BrokenLog";
 	}
+
+	if (!strcasecmp(option, "DumpCore"))
+	{
+		option = OPTION_CRASHDUMP;
+	}
 }
 
 void Options::CheckOptions()
@@ -1644,6 +1745,11 @@ void Options::CheckOptions()
 		LocateOptionSrcPos(OPTION_PARRENAME);
 		ConfigError("Invalid value for option \"%s\": program was compiled without parcheck-support", OPTION_PARRENAME);
 	}
+	if (m_directRename)
+	{
+		LocateOptionSrcPos(OPTION_DIRECTRENAME);
+		ConfigError("Invalid value for option \"%s\": program was compiled without parcheck-support", OPTION_DIRECTRENAME);
+	}
 #endif
 
 #ifdef DISABLE_CURSES
@@ -1660,8 +1766,43 @@ void Options::CheckOptions()
 		LocateOptionSrcPos(OPTION_SECURECONTROL);
 		ConfigError("Invalid value for option \"%s\": program was compiled without TLS/SSL-support", OPTION_SECURECONTROL);
 	}
+
+	if (m_certCheck)
+	{
+		LocateOptionSrcPos(OPTION_CERTCHECK);
+		ConfigError("Invalid value for option \"%s\": program was compiled without TLS/SSL-support", OPTION_CERTCHECK);
+	}
 #endif
 
+#ifdef HAVE_OPENSSL
+#ifndef HAVE_X509_CHECK_HOST
+	if (m_certCheck)
+	{
+		LocateOptionSrcPos(OPTION_CERTCHECK);
+		ConfigWarn("TLS certificate verification (option \"%s\") is limited because the program "
+			"was compiled with older OpenSSL version not supporting hostname validation (at least OpenSSL 1.0.2d is required)", OPTION_CERTCHECK);
+	}
+#endif
+#endif
+
+#ifdef HAVE_LIBGNUTLS
+#if	GNUTLS_VERSION_NUMBER < 0x030104
+	if (m_certCheck)
+	{
+		LocateOptionSrcPos(OPTION_CERTCHECK);
+		ConfigWarn("TLS certificate verification (option \"%s\") is disabled because the program "
+			"was compiled with older GnuTLS version not supporting certificate validation (at least GnuTLS 3.1.4 is required)", OPTION_CERTCHECK);
+	}
+#endif
+#endif
+
+	if (m_certCheck && m_certStore.Empty())
+	{
+		LocateOptionSrcPos(OPTION_CERTCHECK);
+		ConfigError("Option \"%s\" requires proper configuration of option \"%s\"", OPTION_CERTCHECK, OPTION_CERTSTORE);
+		m_certCheck = false;
+	}
+
 	if (!m_decode)
 	{
 		m_directWrite = false;
@@ -1696,7 +1837,7 @@ void Options::CheckOptions()
 	if (sizeof(void*) == 4 && m_parBuffer + m_articleCache > 1900)
 	{
 		ConfigError("Options \"ArticleCache\" and \"ParBuffer\" in total cannot use more than 1900MB of memory in 32-Bit mode. Changed to 1500 and 400");
-		m_articleCache = 1900;
+		m_articleCache = 1500;
 		m_parBuffer = 400;
 	}
 
@@ -1705,3 +1846,77 @@ void Options::CheckOptions()
 		ConfigError("Invalid value for option \"UnpackPassFile\": %s. File not found", *m_unpackPassFile);
 	}
 }
+
+void Options::ConvertOldOptions(OptEntries* optEntries)
+{
+	MergeOldScriptOption(optEntries, OPTION_SCANSCRIPT, true);
+	MergeOldScriptOption(optEntries, OPTION_QUEUESCRIPT, true);
+	MergeOldScriptOption(optEntries, OPTION_FEEDSCRIPT, false);
+}
+
+void Options::MergeOldScriptOption(OptEntries* optEntries, const char* optname, bool mergeCategories)
+{
+	OptEntry* optEntry = optEntries->FindOption(optname);
+	if (!optEntry || Util::EmptyStr(optEntry->GetValue()))
+	{
+		return;
+	}
+
+	OptEntry* extensionsOpt = optEntries->FindOption(OPTION_EXTENSIONS);
+	if (!extensionsOpt)
+	{
+		optEntries->emplace_back(OPTION_EXTENSIONS, "");
+		extensionsOpt = optEntries->FindOption(OPTION_EXTENSIONS);
+	}
+
+	const char* scriptList = optEntry->GetValue();
+
+	Tokenizer tok(scriptList, ",;");
+	while (const char* scriptName = tok.Next())
+	{
+		// merge into global "Extensions"
+		if (!HasScript(extensionsOpt->m_value, scriptName))
+		{
+			if (!extensionsOpt->m_value.Empty())
+			{
+				extensionsOpt->m_value.Append(",");
+			}
+			extensionsOpt->m_value.Append(scriptName);
+		}
+
+		// merge into categories' "Extensions" (if not empty)
+		if (mergeCategories)
+		{
+			for (OptEntry& opt : optEntries)
+			{
+				const char* optname = opt.GetName();
+				if (!strncasecmp(optname, "category", 8))
+				{
+					char* p = (char*)optname + 8;
+					while (*p >= '0' && *p <= '9') p++;
+					if (p && (!strcasecmp(p, ".extensions")))
+					{
+						if (!opt.m_value.Empty() && !HasScript(opt.m_value, scriptName))
+						{
+							opt.m_value.Append(",");
+							opt.m_value.Append(scriptName);
+						}
+					}
+				}
+			}
+		}
+	}
+}
+
+bool Options::HasScript(const char* scriptList, const char* scriptName)
+{
+	Tokenizer tok(scriptList, ",;");
+	while (const char* scriptName2 = tok.Next())
+	{
+		if (!strcasecmp(scriptName2, scriptName))
+		{
+			return true;
+		}
+	}
+	return false;
+};
