@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
- *  Copyright (C) 2008-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2008-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -93,7 +93,10 @@ void Scheduler::CheckTasks()
 
 				for (Task* task : &m_taskList)
 				{
-					task->m_lastExecuted = 0;
+					if (task->m_hours != Task::STARTUP_TASK)
+					{
+						task->m_lastExecuted = 0;
+					}
 				}
 			}
 
@@ -133,9 +136,8 @@ void Scheduler::CheckTasks()
 						}
 
 						bool weekDayOK = task->m_weekDaysBits == 0 || (task->m_weekDaysBits & (1 << (weekDay - 1)));
-						bool doTask = weekDayOK && localLastCheck < appoint && appoint <= localCurrent;
-
-						//debug("TEMP: 1) m_tLastCheck=%i, tLocalCurrent=%i, tLoop=%i, tAppoint=%i, bWeekDayOK=%i, bDoTask=%i", m_tLastCheck, tLocalCurrent, tLoop, tAppoint, (int)bWeekDayOK, (int)bDoTask);
+						bool doTask = (task->m_hours >= 0 && weekDayOK && localLastCheck < appoint && appoint <= localCurrent) ||
+							(task->m_hours == Task::STARTUP_TASK && task->m_lastExecuted == 0);
 
 						if (doTask)
 						{
@@ -162,6 +164,8 @@ void Scheduler::ExecuteTask(Task* task)
 		"Pause Scan", "Unpause Scan", "Enable Server", "Disable Server", "Fetch Feed" };
 	debug("Executing scheduled command: %s", commandName[task->m_command]);
 
+	bool executeProcess = m_executeProcess || task->m_hours == Task::STARTUP_TASK;
+
 	switch (task->m_command)
 	{
 		case scDownloadRate:
@@ -190,9 +194,9 @@ void Scheduler::ExecuteTask(Task* task)
 			m_pauseScanChanged = true;
 			break;
 
-		case scScript:
+		case scExtensions:
 		case scProcess:
-			if (m_executeProcess)
+			if (executeProcess)
 			{
 				SchedulerScriptController::StartScript(task->m_param, task->m_command == scProcess, task->m_id);
 			}
@@ -204,7 +208,7 @@ void Scheduler::ExecuteTask(Task* task)
 			break;
 
 		case scFetchFeed:
-			if (m_executeProcess)
+			if (executeProcess)
 			{
 				FetchFeed(task->m_param);
 				break;
