@@ -2,7 +2,7 @@
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,7 +39,6 @@
 #include "PrePostProcessor.h"
 #include "HistoryCoordinator.h"
 #include "DupeCoordinator.h"
-#include "ParChecker.h"
 #include "Scheduler.h"
 #include "Scanner.h"
 #include "FeedCoordinator.h"
@@ -52,6 +51,7 @@
 #include "Util.h"
 #include "FileSystem.h"
 #include "StackTrace.h"
+#include "CommandScript.h"
 #ifdef WIN32
 #include "WinService.h"
 #include "WinConsole.h"
@@ -60,6 +60,9 @@
 #ifdef ENABLE_TESTS
 #include "TestMain.h"
 #endif
+#ifndef DISABLE_NSERV
+#include "NServMain.h"
+#endif
 
 // Prototypes
 void RunMain();
@@ -82,6 +85,7 @@ ArticleCache* g_ArticleCache;
 QueueScriptCoordinator* g_QueueScriptCoordinator;
 ServiceCoordinator* g_ServiceCoordinator;
 ScriptConfig* g_ScriptConfig;
+CommandScriptLog* g_CommandScriptLog; 
 #ifdef WIN32
 WinConsole* g_WinConsole;
 #endif
@@ -127,6 +131,16 @@ int main(int argc, char *argv[], char *argp[])
 	TestCleanup();
 #endif
 
+	if (argc > 1 && (!strcmp(argv[1], "--nserv")))
+	{
+#ifndef DISABLE_NSERV
+		return NServMain(argc, argv);
+#else
+		printf("ERROR: Could not start NServ, the program was compiled without NServ\n");
+		return 1;
+#endif
+	}
+
 #ifdef WIN32
 	InstallUninstallServiceCheck(argc, argv);
 #endif
@@ -138,6 +152,7 @@ int main(int argc, char *argv[], char *argp[])
 	{
 		if (!strcmp(argv[i], "-D"))
 		{
+			AllocConsole(); // needed for sending CTRL+BREAK signal to child processes
 			StartService(RunMain);
 			return 0;
 		}
@@ -160,7 +175,7 @@ public:
 
 	// Options::Extender
 	virtual void AddNewsServer(int id, bool active, const char* name, const char* host,
-		int port, const char* user, const char* pass, bool joinGroup,
+		int port, int ipVersion, const char* user, const char* pass, bool joinGroup,
 		bool tls, const char* cipher, int maxConnections, int retention,
 		int level, int group, bool optional);
 	virtual void AddFeed(int id, const char* name, const char* url, int interval,
@@ -191,6 +206,7 @@ private:
 	std::unique_ptr<QueueScriptCoordinator> m_queueScriptCoordinator;
 	std::unique_ptr<ServiceCoordinator> m_serviceCoordinator;
 	std::unique_ptr<ScriptConfig> m_scriptConfig;
+	std::unique_ptr<CommandScriptLog> m_commandScriptLog;
 #ifdef WIN32
 	std::unique_ptr<WinConsole> m_winConsole;
 #endif
@@ -266,6 +282,9 @@ void NZBGet::Init()
 
 	m_scanner->InitOptions();
 	m_queueScriptCoordinator->InitOptions();
+#ifndef DISABLE_TLS
+	TlsSocket::InitOptions(g_Options->GetCertCheck() ? g_Options->GetCertStore() : nullptr);
+#endif
 
 	if (m_commandLineParser->GetDaemonMode())
 	{
@@ -351,6 +370,9 @@ void NZBGet::CreateGlobals()
 	m_scriptConfig = std::make_unique<ScriptConfig>();
 	g_ScriptConfig = m_scriptConfig.get();
 
+	m_commandScriptLog = std::make_unique<CommandScriptLog>();
+	g_CommandScriptLog = m_commandScriptLog.get();
+
 	m_scheduler = std::make_unique<Scheduler>();
 
 	m_diskService = std::make_unique<DiskService>();
@@ -368,7 +390,7 @@ void NZBGet::BootConfig()
 	if (m_commandLineParser->GetPrintUsage() || m_commandLineParser->GetErrors() || g_ArgumentCount <= 1)
 	{
 		m_commandLineParser->PrintUsage(((const char**)(*g_Arguments))[0]);
-		exit(0);
+		exit(m_commandLineParser->GetPrintUsage() ? 0 : 1);
 	}
 
 	debug("Reading options");
@@ -394,7 +416,7 @@ void NZBGet::BootConfig()
 	}
 
 	m_serverPool->SetTimeout(m_options->GetArticleTimeout());
-	m_serverPool->SetRetryInterval(m_options->GetRetryInterval());
+	m_serverPool->SetRetryInterval(m_options->GetArticleInterval());
 
 	m_scriptConfig->InitOptions();
 }
@@ -423,6 +445,7 @@ void NZBGet::Cleanup()
 	g_QueueScriptCoordinator = nullptr;
 	g_Maintenance = nullptr;
 	g_StatMeter = nullptr;
+	g_CommandScriptLog = nullptr;
 #ifdef WIN32
 	g_WinConsole = nullptr;
 #endif
@@ -680,106 +703,109 @@ void NZBGet::Run(bool reload)
 void NZBGet::ProcessClientRequest()
 {
 	RemoteClient Client;
+	bool ok = false;
 
 	switch (m_commandLineParser->GetClientOperation())
 	{
 		case CommandLineParser::opClientRequestListFiles:
-			Client.RequestServerList(true, false, m_commandLineParser->GetMatchMode() == CommandLineParser::mmRegEx ? m_commandLineParser->GetEditQueueText() : nullptr);
+			ok = Client.RequestServerList(true, false, m_commandLineParser->GetMatchMode() == CommandLineParser::mmRegEx ? m_commandLineParser->GetEditQueueText() : nullptr);
 			break;
 
 		case CommandLineParser::opClientRequestListGroups:
-			Client.RequestServerList(false, true, m_commandLineParser->GetMatchMode() == CommandLineParser::mmRegEx ? m_commandLineParser->GetEditQueueText() : nullptr);
+			ok = Client.RequestServerList(false, true, m_commandLineParser->GetMatchMode() == CommandLineParser::mmRegEx ? m_commandLineParser->GetEditQueueText() : nullptr);
 			break;
 
 		case CommandLineParser::opClientRequestListStatus:
-			Client.RequestServerList(false, false, nullptr);
+			ok = Client.RequestServerList(false, false, nullptr);
 			break;
 
 		case CommandLineParser::opClientRequestDownloadPause:
-			Client.RequestServerPauseUnpause(true, rpDownload);
+			ok = Client.RequestServerPauseUnpause(true, rpDownload);
 			break;
 
 		case CommandLineParser::opClientRequestDownloadUnpause:
-			Client.RequestServerPauseUnpause(false, rpDownload);
+			ok = Client.RequestServerPauseUnpause(false, rpDownload);
 			break;
 
 		case CommandLineParser::opClientRequestSetRate:
-			Client.RequestServerSetDownloadRate(m_commandLineParser->GetSetRate());
+			ok = Client.RequestServerSetDownloadRate(m_commandLineParser->GetSetRate());
 			break;
 
 		case CommandLineParser::opClientRequestDumpDebug:
-			Client.RequestServerDumpDebug();
+			ok = Client.RequestServerDumpDebug();
 			break;
 
 		case CommandLineParser::opClientRequestEditQueue:
-			Client.RequestServerEditQueue((DownloadQueue::EEditAction)m_commandLineParser->GetEditQueueAction(),
+			ok = Client.RequestServerEditQueue((DownloadQueue::EEditAction)m_commandLineParser->GetEditQueueAction(),
 				m_commandLineParser->GetEditQueueOffset(), m_commandLineParser->GetEditQueueText(),
 				m_commandLineParser->GetEditQueueIdList(), m_commandLineParser->GetEditQueueNameList(),
 				(ERemoteMatchMode)m_commandLineParser->GetMatchMode());
 			break;
 
 		case CommandLineParser::opClientRequestLog:
-			Client.RequestServerLog(m_commandLineParser->GetLogLines());
+			ok = Client.RequestServerLog(m_commandLineParser->GetLogLines());
 			break;
 
 		case CommandLineParser::opClientRequestShutdown:
-			Client.RequestServerShutdown();
+			ok = Client.RequestServerShutdown();
 			break;
 
 		case CommandLineParser::opClientRequestReload:
-			Client.RequestServerReload();
+			ok = Client.RequestServerReload();
 			break;
 
 		case CommandLineParser::opClientRequestDownload:
-			Client.RequestServerDownload(m_commandLineParser->GetAddNzbFilename(), m_commandLineParser->GetArgFilename(),
+			ok = Client.RequestServerDownload(m_commandLineParser->GetAddNzbFilename(), m_commandLineParser->GetArgFilename(),
 				m_commandLineParser->GetAddCategory(), m_commandLineParser->GetAddTop(), m_commandLineParser->GetAddPaused(), m_commandLineParser->GetAddPriority(),
 				m_commandLineParser->GetAddDupeKey(), m_commandLineParser->GetAddDupeMode(), m_commandLineParser->GetAddDupeScore());
 			break;
 
 		case CommandLineParser::opClientRequestVersion:
-			Client.RequestServerVersion();
+			ok = Client.RequestServerVersion();
 			break;
 
 		case CommandLineParser::opClientRequestPostQueue:
-			Client.RequestPostQueue();
+			ok = Client.RequestPostQueue();
 			break;
 
 		case CommandLineParser::opClientRequestWriteLog:
-			Client.RequestWriteLog(m_commandLineParser->GetWriteLogKind(), m_commandLineParser->GetLastArg());
+			ok = Client.RequestWriteLog(m_commandLineParser->GetWriteLogKind(), m_commandLineParser->GetLastArg());
 			break;
 
 		case CommandLineParser::opClientRequestScanAsync:
-			Client.RequestScan(false);
+			ok = Client.RequestScan(false);
 			break;
 
 		case CommandLineParser::opClientRequestScanSync:
-			Client.RequestScan(true);
+			ok = Client.RequestScan(true);
 			break;
 
 		case CommandLineParser::opClientRequestPostPause:
-			Client.RequestServerPauseUnpause(true, rpPostProcess);
+			ok = Client.RequestServerPauseUnpause(true, rpPostProcess);
 			break;
 
 		case CommandLineParser::opClientRequestPostUnpause:
-			Client.RequestServerPauseUnpause(false, rpPostProcess);
+			ok = Client.RequestServerPauseUnpause(false, rpPostProcess);
 			break;
 
 		case CommandLineParser::opClientRequestScanPause:
-			Client.RequestServerPauseUnpause(true, rpScan);
+			ok = Client.RequestServerPauseUnpause(true, rpScan);
 			break;
 
 		case CommandLineParser::opClientRequestScanUnpause:
-			Client.RequestServerPauseUnpause(false, rpScan);
+			ok = Client.RequestServerPauseUnpause(false, rpScan);
 			break;
 
 		case CommandLineParser::opClientRequestHistory:
 		case CommandLineParser::opClientRequestHistoryAll:
-			Client.RequestHistory(m_commandLineParser->GetClientOperation() == CommandLineParser::opClientRequestHistoryAll);
+			ok = Client.RequestHistory(m_commandLineParser->GetClientOperation() == CommandLineParser::opClientRequestHistoryAll);
 			break;
 
 		case CommandLineParser::opClientNoOperation:
-			break;
+			return;
 	}
+
+	exit(ok ? 0 : 1);
 }
 
 void NZBGet::ProcessWebGet()
@@ -865,16 +891,12 @@ void NZBGet::Daemonize()
 	// obtain a new process group
 	setsid();
 
-	// close all descriptors
-	for (int i = getdtablesize(); i >= 0; --i)
-	{
-		close(i);
-	}
-
 	// handle standart I/O
 	int d = open("/dev/null", O_RDWR);
-	dup(d);
-	dup(d);
+	dup2(d, 0);
+	dup2(d, 1);
+	dup2(d, 2);
+	close(d);
 
 	// set up lock-file
 	int lfp = -1;
@@ -928,10 +950,10 @@ void NZBGet::Daemonize()
 #endif
 
 void NZBGet::AddNewsServer(int id, bool active, const char* name, const char* host,
-	int port, const char* user, const char* pass, bool joinGroup, bool tls,
+	int port, int ipVersion, const char* user, const char* pass, bool joinGroup, bool tls,
 	const char* cipher, int maxConnections, int retention, int level, int group, bool optional)
 {
-	m_serverPool->AddServer(std::make_unique<NewsServer>(id, active, name, host, port, user, pass, joinGroup,
+	m_serverPool->AddServer(std::make_unique<NewsServer>(id, active, name, host, port, ipVersion, user, pass, joinGroup,
 		tls, cipher, maxConnections, retention, level, group, optional));
 }
 
