@@ -40,6 +40,9 @@
 //#define HAVE_LIBGNUTLS
 #endif
 
+/* Define to 1 if OpenSSL supports function "X509_check_host". */
+#define HAVE_X509_CHECK_HOST 1
+
 /* Define to the name of macro which returns the name of function being
 compiled */
 #define FUNCTION_MACRO_NAME __FUNCTION__
@@ -196,6 +199,7 @@ using namespace MSXML;
 #include <iostream>
 #include <fstream>
 #include <memory>
+#include <functional>
 
 #ifdef HAVE_LIBGNUTLS
 #ifdef WIN32
@@ -204,12 +208,16 @@ typedef SSIZE_T ssize_t;
 typedef int pid_t;
 #endif
 #include <gnutls/gnutls.h>
+#include <gnutls/x509.h>
 #if GNUTLS_VERSION_NUMBER <= 0x020b00
 #define NEED_GCRYPT_LOCKING
 #endif
 #ifdef NEED_GCRYPT_LOCKING
 #include <gcrypt.h>
 #endif /* NEED_GCRYPT_LOCKING */
+#include <nettle/sha.h>
+#include <nettle/pbkdf2.h>
+#include <nettle/aes.h>
 #endif /* HAVE_LIBGNUTLS */
 
 #ifdef HAVE_OPENSSL
@@ -218,6 +226,7 @@ typedef int pid_t;
 #include <openssl/rsa.h>
 #include <openssl/sha.h>
 #include <openssl/pem.h>
+#include <openssl/x509v3.h>
 #endif /* HAVE_OPENSSL */
 
 #ifdef HAVE_REGEX_H
@@ -318,12 +327,16 @@ typedef int pid_t;
 
 #ifdef HAVE_STDINT_H
 typedef uint8_t uint8;
+typedef int16_t int16;
+typedef uint16_t uint16;
 typedef uint32_t int32;
 typedef uint32_t uint32;
 typedef int64_t int64;
 typedef uint64_t uint64;
 #else
 typedef unsigned char uint8;
+typedef signed short int16;
+typedef unsigned short uint16;
 typedef signed int int32;
 typedef unsigned int uint32;
 typedef signed long long int64;
