@@ -63,8 +63,8 @@ void ArticleDownloader::SetInfoName(const char* infoName)
 		- if download fails with error "Not-Found" (article or group not found) or with CRC error,
 		  add the server to failed server list;
 		- if download fails with general failure error (article incomplete, other unknown error
-		  codes), try the same server again as many times as defined by option <Retries>; if all attempts
-		  fail, add the server to failed server list;
+		  codes), try the same server again as many times as defined by option <ArticleRetries>;
+		  if all attempts fail, add the server to failed server list;
 		- if all servers from current level were tried, increase level;
 		- if all servers from all levels were tried, break the loop with failure status.
 	<end-loop>
@@ -80,7 +80,7 @@ void ArticleDownloader::Run()
 	m_articleWriter.Prepare();
 
 	EStatus status = adFailed;
-	int retries = g_Options->GetRetries() > 0 ? g_Options->GetRetries() : 1;
+	int retries = g_Options->GetArticleRetries() > 0 ? g_Options->GetArticleRetries() : 1;
 	int remainedRetries = retries;
 	ServerPool::RawServerList failedServers;
 	failedServers.reserve(g_ServerPool->GetServers()->size());
@@ -296,6 +296,11 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 	m_writingStarted = false;
 	m_articleInfo->SetCrc(0);
 
+	if (m_contentAnalyzer)
+	{
+		m_contentAnalyzer->Reset();
+	}
+
 	if (m_connection->GetNewsServer()->GetJoinGroup())
 	{
 		// change group
@@ -316,14 +321,7 @@ ArticleDownloader::EStatus ArticleDownloader::Download()
 	}
 
 	// retrieve article
-	for (int retry = 3; retry > 0; retry--)
-	{
-		response = m_connection->Request(BString<1024>("ARTICLE %s\r\n", m_articleInfo->GetMessageId()));
-		if ((response && !strncmp(response, "2", 1)) || m_connection->GetAuthError())
-		{
-			break;
-		}
-	}
+	response = m_connection->Request(BString<1024>("ARTICLE %s\r\n", m_articleInfo->GetMessageId()));
 
 	status = CheckResponse(response, "could not fetch article");
 	if (status != adFinished)
@@ -548,6 +546,11 @@ bool ArticleDownloader::Write(char* line, int len)
 
 	bool ok = len == 0 || m_articleWriter.Write(line, len);
 
+	if (m_contentAnalyzer)
+	{
+		m_contentAnalyzer->Append(line, len);
+	}
+
 	return ok;
 }
 
