@@ -84,7 +84,7 @@ bool ArticleWriter::Start(Decoder::EFormat format, const char* filename, int64 f
 			}
 
 			if (!outputInitialized && filename &&
-				FileSystem::FileExists(m_fileInfo->GetNzbInfo()->GetDestDir(), filename))
+				FileSystem::FileExists(BString<1024>("%s%c%s", m_fileInfo->GetNzbInfo()->GetDestDir(), PATH_SEPARATOR, filename)))
 			{
 				m_duplicate = true;
 				return false;
@@ -275,7 +275,7 @@ void ArticleWriter::BuildOutputFilename()
 		else
 		{
 			filename.Format("%s%c%i.out.tmp", m_fileInfo->GetNzbInfo()->GetDestDir(),
-				(int)PATH_SEPARATOR, m_fileInfo->GetId());
+				PATH_SEPARATOR, m_fileInfo->GetId());
 			m_fileInfo->SetOutputFilename(filename);
 		}
 
@@ -292,14 +292,16 @@ void ArticleWriter::CompleteFileParts()
 
 	BString<1024> nzbName;
 	BString<1024> nzbDestDir;
+	BString<1024> filename;
 
 	{
 		GuardedDownloadQueue guard = DownloadQueue::Guard();
 		nzbName = m_fileInfo->GetNzbInfo()->GetName();
 		nzbDestDir = m_fileInfo->GetNzbInfo()->GetDestDir();
+		filename = m_fileInfo->GetFilename();
 	}
 
-	BString<1024> infoFilename("%s%c%s", *nzbName, (int)PATH_SEPARATOR, m_fileInfo->GetFilename());
+	BString<1024> infoFilename("%s%c%s", *nzbName, PATH_SEPARATOR, *filename);
 
 	bool cached = m_fileInfo->GetCachedArticles() > 0;
 
@@ -332,11 +334,11 @@ void ArticleWriter::CompleteFileParts()
 	CString ofn;
 	if (m_fileInfo->GetForceDirectWrite())
 	{
-		ofn.Format("%s%c%s", *nzbDestDir, PATH_SEPARATOR, m_fileInfo->GetFilename());
+		ofn.Format("%s%c%s", *nzbDestDir, PATH_SEPARATOR, *filename);
 	}
 	else
 	{
-		ofn = FileSystem::MakeUniqueFilename(nzbDestDir, m_fileInfo->GetFilename());
+		ofn = FileSystem::MakeUniqueFilename(nzbDestDir, *filename);
 	}
 
 	DiskFile outfile;
@@ -436,14 +438,14 @@ void ArticleWriter::CompleteFileParts()
 					m_fileInfo->SetFailedArticles(m_fileInfo->GetFailedArticles() + 1);
 					m_fileInfo->SetSuccessArticles(m_fileInfo->GetSuccessArticles() - 1);
 					m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
-						"Could not find file %s for %s%c%s [%i/%i]",
-						pa->GetResultFilename(), *nzbName, (int)PATH_SEPARATOR, m_fileInfo->GetFilename(),
-						pa->GetPartNumber(), (int)m_fileInfo->GetArticles()->size());
+						"Could not find file %s for %s [%i/%i]",
+						pa->GetResultFilename(), *infoFilename, pa->GetPartNumber(),
+						(int)m_fileInfo->GetArticles()->size());
 				}
 			}
 			else if (!g_Options->GetDecode())
 			{
-				BString<1024> dstFileName("%s%c%03i", *ofn, (int)PATH_SEPARATOR, pa->GetPartNumber());
+				BString<1024> dstFileName("%s%c%03i", *ofn, PATH_SEPARATOR, pa->GetPartNumber());
 				if (!FileSystem::MoveFile(pa->GetResultFilename(), dstFileName))
 				{
 					m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
@@ -523,11 +525,11 @@ void ArticleWriter::CompleteFileParts()
 
 		if (g_Options->GetBrokenLog())
 		{
-			BString<1024> brokenLogName("%s%c_brokenlog.txt", *nzbDestDir, (int)PATH_SEPARATOR);
+			BString<1024> brokenLogName("%s%c_brokenlog.txt", *nzbDestDir, PATH_SEPARATOR);
 			DiskFile file;
 			if (file.Open(brokenLogName, DiskFile::omAppend))
 			{
-				file.Print("%s (%i/%i)%s", m_fileInfo->GetFilename(), m_fileInfo->GetSuccessArticles(),
+				file.Print("%s (%i/%i)%s", *filename, m_fileInfo->GetSuccessArticles(),
 					m_fileInfo->GetTotalArticles(), LINE_ENDING);
 				file.Close();
 			}
@@ -538,11 +540,25 @@ void ArticleWriter::CompleteFileParts()
 		m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkInfo, "Partially downloaded %s", *infoFilename);
 	}
 
-	m_fileInfo->SetCrc(crc);
-	m_fileInfo->SetOutputFilename(ofn);
-
 	{
 		GuardedDownloadQueue guard = DownloadQueue::Guard();
+
+		m_fileInfo->SetCrc(crc);
+		m_fileInfo->SetOutputFilename(ofn);
+
+		if (strcmp(m_fileInfo->GetFilename(), filename))
+		{
+			// file was renamed during completion, need to move the file
+			ofn = FileSystem::MakeUniqueFilename(nzbDestDir, m_fileInfo->GetFilename());
+			if (!FileSystem::MoveFile(m_fileInfo->GetOutputFilename(), ofn))
+			{
+				m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
+					"Could not rename file %s to %s: %s", m_fileInfo->GetOutputFilename(),
+					*ofn, *FileSystem::GetLastErrorMessage());
+			}
+			m_fileInfo->SetOutputFilename(ofn);
+		}
+
 		if (strcmp(m_fileInfo->GetNzbInfo()->GetDestDir(), nzbDestDir))
 		{
 			// destination directory was changed during completion, need to move the file
@@ -593,6 +609,9 @@ void ArticleWriter::FlushCache()
 					m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
 						"Could not open file %s: %s", m_fileInfo->GetOutputFilename(),
 						*FileSystem::GetLastErrorMessage());
+					// prevent multiple error messages
+					pa->DiscardSegment();
+					flushedArticles++;
 					break;
 				}
 				needBufFile = true;
@@ -608,6 +627,9 @@ void ArticleWriter::FlushCache()
 					m_fileInfo->GetNzbInfo()->PrintMessage(Message::mkError,
 						"Could not create file %s: %s", *destFile,
 						*FileSystem::GetLastErrorMessage());
+					// prevent multiple error messages
+					pa->DiscardSegment();
+					flushedArticles++;
 					break;
 				}
 				needBufFile = true;
@@ -674,14 +696,14 @@ bool ArticleWriter::MoveCompletedFiles(NzbInfo* nzbInfo, const char* oldDestDir)
 	// move already downloaded files to new destination
 	for (CompletedFile& completedFile : nzbInfo->GetCompletedFiles())
 	{
-		BString<1024> oldFileName("%s%c%s", oldDestDir, (int)PATH_SEPARATOR, completedFile.GetFileName());
-		BString<1024> newFileName("%s%c%s", nzbInfo->GetDestDir(), (int)PATH_SEPARATOR, completedFile.GetFileName());
+		BString<1024> oldFileName("%s%c%s", oldDestDir, PATH_SEPARATOR, completedFile.GetFilename());
+		BString<1024> newFileName("%s%c%s", nzbInfo->GetDestDir(), PATH_SEPARATOR, completedFile.GetFilename());
 
 		// check if file was not moved already
 		if (strcmp(oldFileName, newFileName))
 		{
 			// prevent overwriting of existing files
-			newFileName = FileSystem::MakeUniqueFilename(nzbInfo->GetDestDir(), completedFile.GetFileName());
+			newFileName = FileSystem::MakeUniqueFilename(nzbInfo->GetDestDir(), completedFile.GetFilename());
 
 			detail("Moving file %s to %s", *oldFileName, *newFileName);
 			if (!FileSystem::MoveFile(oldFileName, newFileName))
@@ -695,10 +717,10 @@ bool ArticleWriter::MoveCompletedFiles(NzbInfo* nzbInfo, const char* oldDestDir)
 	// move brokenlog.txt
 	if (g_Options->GetBrokenLog())
 	{
-		BString<1024> oldBrokenLogName("%s%c_brokenlog.txt", oldDestDir, (int)PATH_SEPARATOR);
+		BString<1024> oldBrokenLogName("%s%c_brokenlog.txt", oldDestDir, PATH_SEPARATOR);
 		if (FileSystem::FileExists(oldBrokenLogName))
 		{
-			BString<1024> brokenLogName("%s%c_brokenlog.txt", nzbInfo->GetDestDir(), (int)PATH_SEPARATOR);
+			BString<1024> brokenLogName("%s%c_brokenlog.txt", nzbInfo->GetDestDir(), PATH_SEPARATOR);
 
 			detail("Moving file %s to %s", *oldBrokenLogName, *brokenLogName);
 			if (FileSystem::FileExists(brokenLogName))
@@ -871,7 +893,7 @@ bool ArticleCache::CheckFlush(bool flushEverything)
 				if (fileInfo->GetCachedArticles() > 0 && (fileInfo->GetActiveDownloads() == 0 || flushEverything))
 				{
 					m_fileInfo = fileInfo;
-					infoName.Format("%s%c%s", m_fileInfo->GetNzbInfo()->GetName(), (int)PATH_SEPARATOR, m_fileInfo->GetFilename());
+					infoName.Format("%s%c%s", m_fileInfo->GetNzbInfo()->GetName(), PATH_SEPARATOR, m_fileInfo->GetFilename());
 					break;
 				}
 			}
