@@ -1,7 +1,7 @@
 /*
- *  This file is part of nzbget
+ *  This file is part of nzbget. See <http://nzbget.net>.
  *
- *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,30 +14,10 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *
- * $Revision$
- * $Date$
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#ifdef WIN32
-#include "win32.h"
-#endif
-
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
-
 #include "nzbget.h"
 #include "Decoder.h"
 #include "Log.h"
@@ -45,24 +25,9 @@
 
 const char* Decoder::FormatNames[] = { "Unknown", "yEnc", "UU" };
 
-Decoder::Decoder()
-{
-	debug("Creating Decoder");
-
-	m_szArticleFilename	= NULL;
-}
-
-Decoder::~ Decoder()
-{
-	debug("Destroying Decoder");
-
-	free(m_szArticleFilename);
-}
-
 void Decoder::Clear()
 {
-	free(m_szArticleFilename);
-	m_szArticleFilename = NULL;
+	m_articleFilename.Clear();
 }
 
 Decoder::EFormat Decoder::DetectFormat(const char* buffer, int len, bool inBody)
@@ -71,7 +36,7 @@ Decoder::EFormat Decoder::DetectFormat(const char* buffer, int len, bool inBody)
 	{
 		return efYenc;
 	}
-	
+
 	if (inBody && (len == 62 || len == 63) && (buffer[62] == '\n' || buffer[62] == '\r') && *buffer == 'M')
 	{
 		return efUx;
@@ -79,18 +44,18 @@ Decoder::EFormat Decoder::DetectFormat(const char* buffer, int len, bool inBody)
 
 	if (!strncmp(buffer, "begin ", 6))
 	{
-		bool bOK = true;
+		bool ok = true;
 		buffer += 6; //strlen("begin ")
 		while (*buffer && *buffer != ' ')
 		{
 			char ch = *buffer++;
 			if (ch < '0' || ch > '7')
 			{
-				bOK = false;
+				ok = false;
 				break;
 			}
 		}
-		if (bOK)
+		if (ok)
 		{
 			return efUx;
 		}
@@ -112,39 +77,39 @@ void YDecoder::Clear()
 {
 	Decoder::Clear();
 
-	m_bBody = false;
-	m_bBegin = false;
-	m_bPart = false;
-	m_bEnd = false;
-	m_bCrc = false;
-	m_lExpectedCRC = 0;
-	m_lCalculatedCRC = 0xFFFFFFFF;
-	m_iBegin = 0;
-	m_iEnd = 0;
-	m_iSize = 0;
-	m_iEndSize = 0;
-	m_bCrcCheck = false;
+	m_body = false;
+	m_begin = false;
+	m_part = false;
+	m_end = false;
+	m_crc = false;
+	m_expectedCRC = 0;
+	m_calculatedCRC = 0xFFFFFFFF;
+	m_beginPos = 0;
+	m_endPos = 0;
+	m_size = 0;
+	m_endSize = 0;
+	m_crcCheck = false;
 }
 
 int YDecoder::DecodeBuffer(char* buffer, int len)
 {
-	if (m_bBody && !m_bEnd)
+	if (m_body && !m_end)
 	{
 		if (!strncmp(buffer, "=yend ", 6))
 		{
-			m_bEnd = true;
-			char* pb = strstr(buffer, m_bPart ? " pcrc32=" : " crc32=");
+			m_end = true;
+			char* pb = strstr(buffer, m_part ? " pcrc32=" : " crc32=");
 			if (pb)
 			{
-				m_bCrc = true;
-				pb += 7 + (int)m_bPart; //=strlen(" crc32=") or strlen(" pcrc32=")
-				m_lExpectedCRC = strtoul(pb, NULL, 16);
+				m_crc = true;
+				pb += 7 + (int)m_part; //=strlen(" crc32=") or strlen(" pcrc32=")
+				m_expectedCRC = strtoul(pb, nullptr, 16);
 			}
 			pb = strstr(buffer, " size=");
-			if (pb) 
+			if (pb)
 			{
 				pb += 6; //=strlen(" size=")
-				m_iEndSize = (long long)atoll(pb);
+				m_endSize = (int64)atoll(pb);
 			}
 			return 0;
 		}
@@ -174,57 +139,54 @@ int YDecoder::DecodeBuffer(char* buffer, int len)
 		}
 BreakLoop:
 
-		if (m_bCrcCheck)
+		if (m_crcCheck)
 		{
-			m_lCalculatedCRC = Util::Crc32m(m_lCalculatedCRC, (unsigned char *)buffer, (unsigned int)(optr - buffer));
+			m_calculatedCRC = Util::Crc32m(m_calculatedCRC, (uchar *)buffer, (uint32)(optr - buffer));
 		}
 		return optr - buffer;
 	}
-	else 
+	else
 	{
-		if (!m_bPart && !strncmp(buffer, "=ybegin ", 8))
+		if (!m_part && !strncmp(buffer, "=ybegin ", 8))
 		{
-			m_bBegin = true;
+			m_begin = true;
 			char* pb = strstr(buffer, " name=");
 			if (pb)
 			{
 				pb += 6; //=strlen(" name=")
 				char* pe;
 				for (pe = pb; *pe != '\0' && *pe != '\n' && *pe != '\r'; pe++) ;
-				free(m_szArticleFilename);
-				m_szArticleFilename = (char*)malloc(pe - pb + 1);
-				strncpy(m_szArticleFilename, pb, pe - pb);
-				m_szArticleFilename[pe - pb] = '\0';
+				m_articleFilename = WebUtil::Latin1ToUtf8(CString(pb, pe - pb));
 			}
 			pb = strstr(buffer, " size=");
-			if (pb) 
+			if (pb)
 			{
 				pb += 6; //=strlen(" size=")
-				m_iSize = (long long)atoll(pb);
+				m_size = (int64)atoll(pb);
 			}
-			m_bPart = strstr(buffer, " part=");
-			if (!m_bPart)
+			m_part = strstr(buffer, " part=");
+			if (!m_part)
 			{
-				m_bBody = true;
-				m_iBegin = 1;
-				m_iEnd = m_iSize;
+				m_body = true;
+				m_beginPos = 1;
+				m_endPos = m_size;
 			}
 		}
-		else if (m_bPart && !strncmp(buffer, "=ypart ", 7))
+		else if (m_part && !strncmp(buffer, "=ypart ", 7))
 		{
-			m_bPart = true;
-			m_bBody = true;
+			m_part = true;
+			m_body = true;
 			char* pb = strstr(buffer, " begin=");
-			if (pb) 
+			if (pb)
 			{
 				pb += 7; //=strlen(" begin=")
-				m_iBegin = (long long)atoll(pb);
+				m_beginPos = (int64)atoll(pb);
 			}
 			pb = strstr(buffer, " end=");
-			if (pb) 
+			if (pb)
 			{
 				pb += 5; //=strlen(" end=")
-				m_iEnd = (long long)atoll(pb);
+				m_endPos = (int64)atoll(pb);
 			}
 		}
 	}
@@ -234,29 +196,29 @@ BreakLoop:
 
 Decoder::EStatus YDecoder::Check()
 {
-	m_lCalculatedCRC ^= 0xFFFFFFFF;
+	m_calculatedCRC ^= 0xFFFFFFFF;
 
-	debug("Expected crc32=%x", m_lExpectedCRC);
-	debug("Calculated crc32=%x", m_lCalculatedCRC);
+	debug("Expected crc32=%x", m_expectedCRC);
+	debug("Calculated crc32=%x", m_calculatedCRC);
 
-	if (!m_bBegin)
+	if (!m_begin)
 	{
-		return eNoBinaryData;
+		return dsNoBinaryData;
 	}
-	else if (!m_bEnd)
+	else if (!m_end)
 	{
-		return eArticleIncomplete;
+		return dsArticleIncomplete;
 	}
-	else if (!m_bPart && m_iSize != m_iEndSize)
+	else if (!m_part && m_size != m_endSize)
 	{
-		return eInvalidSize;
+		return dsInvalidSize;
 	}
-	else if (m_bCrcCheck && m_bCrc && (m_lExpectedCRC != m_lCalculatedCRC))
+	else if (m_crcCheck && m_crc && (m_expectedCRC != m_calculatedCRC))
 	{
-		return eCrcError;
+		return dsCrcError;
 	}
 
-	return eFinished;
+	return dsFinished;
 }
 
 
@@ -266,15 +228,15 @@ Decoder::EStatus YDecoder::Check()
 
 UDecoder::UDecoder()
 {
-
+	Clear();
 }
 
 void UDecoder::Clear()
 {
 	Decoder::Clear();
 
-	m_bBody = false;
-	m_bEnd = false;
+	m_body = false;
+	m_end = false;
 }
 
 /* DecodeBuffer-function uses portions of code from tool UUDECODE by Clem Dye
@@ -288,7 +250,7 @@ void UDecoder::Clear()
 
 int UDecoder::DecodeBuffer(char* buffer, int len)
 {
-	if (!m_bBody)
+	if (!m_body)
 	{
 		if (!strncmp(buffer, "begin ", 6))
 		{
@@ -296,35 +258,32 @@ int UDecoder::DecodeBuffer(char* buffer, int len)
 			pb += 6; //strlen("begin ")
 
 			// skip file-permissions
-			for (; *pb != ' ' && *pb != '\0' && *pb != '\n' && *pb != '\r'; pb++) ; 
+			for (; *pb != ' ' && *pb != '\0' && *pb != '\n' && *pb != '\r'; pb++) ;
 			pb++;
 
 			// extracting filename
 			char* pe;
 			for (pe = pb; *pe != '\0' && *pe != '\n' && *pe != '\r'; pe++) ;
-			free(m_szArticleFilename);
-			m_szArticleFilename = (char*)malloc(pe - pb + 1);
-			strncpy(m_szArticleFilename, pb, pe - pb);
-			m_szArticleFilename[pe - pb] = '\0';
+			m_articleFilename = WebUtil::Latin1ToUtf8(CString(pb, pe - pb));
 
-			m_bBody = true;
+			m_body = true;
 			return 0;
 		}
 		else if ((len == 62 || len == 63) && (buffer[62] == '\n' || buffer[62] == '\r') && *buffer == 'M')
 		{
-			m_bBody = true;
+			m_body = true;
 		}
 	}
 
-	if (m_bBody && (!strncmp(buffer, "end ", 4) || *buffer == '`'))
+	if (m_body && (!strncmp(buffer, "end ", 4) || *buffer == '`'))
 	{
-		m_bEnd = true;
+		m_end = true;
 	}
 
-	if (m_bBody && !m_bEnd)
+	if (m_body && !m_end)
 	{
-		int iEffLen = UU_DECODE_CHAR(buffer[0]);
-		if (iEffLen > len)
+		int effLen = UU_DECODE_CHAR(buffer[0]);
+		if (effLen > len)
 		{
 			// error;
 			return 0;
@@ -332,23 +291,23 @@ int UDecoder::DecodeBuffer(char* buffer, int len)
 
 		char* iptr = buffer;
 		char* optr = buffer;
-		for (++iptr; iEffLen > 0; iptr += 4, iEffLen -= 3)
+		for (++iptr; effLen > 0; iptr += 4, effLen -= 3)
 		{
-			if (iEffLen >= 3)
+			if (effLen >= 3)
 			{
-				*optr++ = UU_DECODE_CHAR (iptr[0]) << 2 | UU_DECODE_CHAR (iptr[1]) >> 4; 
-				*optr++ = UU_DECODE_CHAR (iptr[1]) << 4 | UU_DECODE_CHAR (iptr[2]) >> 2; 
+				*optr++ = UU_DECODE_CHAR (iptr[0]) << 2 | UU_DECODE_CHAR (iptr[1]) >> 4;
+				*optr++ = UU_DECODE_CHAR (iptr[1]) << 4 | UU_DECODE_CHAR (iptr[2]) >> 2;
 				*optr++ = UU_DECODE_CHAR (iptr[2]) << 6 | UU_DECODE_CHAR (iptr[3]);
 			}
 			else
 			{
-				if (iEffLen >= 1)
+				if (effLen >= 1)
 				{
-					*optr++ = UU_DECODE_CHAR (iptr[0]) << 2 | UU_DECODE_CHAR (iptr[1]) >> 4; 
+					*optr++ = UU_DECODE_CHAR (iptr[0]) << 2 | UU_DECODE_CHAR (iptr[1]) >> 4;
 				}
-				if (iEffLen >= 2)
+				if (effLen >= 2)
 				{
-					*optr++ = UU_DECODE_CHAR (iptr[1]) << 4 | UU_DECODE_CHAR (iptr[2]) >> 2; 
+					*optr++ = UU_DECODE_CHAR (iptr[1]) << 4 | UU_DECODE_CHAR (iptr[2]) >> 2;
 				}
 			}
 		}
@@ -361,10 +320,10 @@ int UDecoder::DecodeBuffer(char* buffer, int len)
 
 Decoder::EStatus UDecoder::Check()
 {
-	if (!m_bBody)
+	if (!m_body)
 	{
-		return eNoBinaryData;
+		return dsNoBinaryData;
 	}
 
-	return eFinished;
+	return dsFinished;
 }
