@@ -2,7 +2,7 @@
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -470,13 +470,20 @@ void NzbInfo::SetActiveDownloads(int activeDownloads)
 		if (activeDownloads > 0)
 		{
 			m_downloadStartTime = Util::CurrentTime();
+			m_downloadStartSec = m_downloadSec;
 		}
 		else
 		{
-			m_downloadSec += Util::CurrentTime() - m_downloadStartTime;
+			m_downloadSec = m_downloadStartSec + (Util::CurrentTime() - m_downloadStartTime);
 			m_downloadStartTime = 0;
+			m_changed = true;
 		}
 	}
+	else if (activeDownloads > 0)
+	{
+		m_downloadSec = m_downloadStartSec + (Util::CurrentTime() - m_downloadStartTime);
+		m_changed = true;
+	}
 	m_activeDownloads = activeDownloads;
 }
 
@@ -725,6 +732,24 @@ void NzbInfo::UpdateDeletedStats(FileInfo* fileInfo)
 	m_currentServerStats.ListOp(fileInfo->GetServerStats(), ServerStatList::soSubtract);
 }
 
+bool NzbInfo::IsDownloadCompleted(bool ignorePausedPars)
+{
+	if (m_activeDownloads)
+	{
+		return false;
+	}
+
+	for (FileInfo* fileInfo : &m_fileList)
+	{
+		if ((!fileInfo->GetPaused() || !ignorePausedPars || !fileInfo->GetParFile()) &&
+			!fileInfo->GetDeleted())
+		{
+			return false;
+		}
+	}
+
+	return true;
+}
 
 void ArticleInfo::AttachSegment(std::unique_ptr<SegmentData> content, int64 offset, int size)
 {
@@ -791,9 +816,10 @@ void FileInfo::SetActiveDownloads(int activeDownloads)
 }
 
 
-CompletedFile::CompletedFile(int id, const char* fileName, EStatus status, uint32 crc) :
-	m_id(id), m_fileName(fileName), m_status(status), m_crc(crc)
-
+CompletedFile::CompletedFile(int id, const char* filename, EStatus status, uint32 crc,
+	bool parFile, const char* hash16k, const char* parSetId) :
+	m_id(id), m_filename(filename), m_status(status), m_crc(crc), m_parFile(parFile),
+	m_hash16k(hash16k), m_parSetId(parSetId)
 {
 	if (FileInfo::m_idMax < m_id)
 	{
