@@ -2,7 +2,7 @@
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -192,6 +192,11 @@ public:
 	void SetPartialState(EPartialState partialState) { m_partialState = partialState; }
 	uint32 GetCrc() { return m_crc; }
 	void SetCrc(uint32 crc) { m_crc = crc; }
+	const char* GetHash16k() { return m_hash16k; }
+	void SetHash16k(const char* hash16k) { m_hash16k = hash16k; }
+	const char* GetParSetId() { return m_parSetId; }
+	void SetParSetId(const char* parSetId) { m_parSetId = parSetId; }
+
 	ServerStatList* GetServerStats() { return &m_serverStats; }
 
 private:
@@ -228,6 +233,8 @@ private:
 	bool m_forceDirectWrite = false;
 	EPartialState m_partialState = psNone;
 	uint32 m_crc = 0;
+	CString m_hash16k;
+	CString m_parSetId;
 
 	static int m_idGen;
 	static int m_idMax;
@@ -249,18 +256,27 @@ public:
 		cfFailure
 	};
 
-	CompletedFile(int id, const char* fileName, EStatus status, uint32 crc);
+	CompletedFile(int id, const char* filename, EStatus status, uint32 crc, 
+		bool parFile, const char* hash16k, const char* parSetId);
 	int GetId() { return m_id; }
-	void SetFileName(const char* fileName) { m_fileName = fileName; }
-	const char* GetFileName() { return m_fileName; }
+	void SetFilename(const char* filename) { m_filename = filename; }
+	const char* GetFilename() { return m_filename; }
+	bool GetParFile() { return m_parFile; }
 	EStatus GetStatus() { return m_status; }
 	uint32 GetCrc() { return m_crc; }
+	const char* GetHash16k() { return m_hash16k; }
+	void SetHash16k(const char* hash16k) { m_hash16k = hash16k; }
+	const char* GetParSetId() { return m_parSetId; }
+	void SetParSetId(const char* parSetId) { m_parSetId = parSetId; }
 
 private:
 	int m_id;
-	CString m_fileName;
+	CString m_filename;
 	EStatus m_status;
 	uint32 m_crc;
+	bool m_parFile;
+	CString m_hash16k;
+	CString m_parSetId;
 };
 
 typedef std::deque<CompletedFile> CompletedFileList;
@@ -332,11 +348,19 @@ enum EDupeMode
 class NzbInfo
 {
 public:
-	enum ERenameStatus
+	enum EDirectRenameStatus
+	{
+		tsNone,
+		tsRunning,
+		tsFailure,
+		tsSuccess
+	};
+
+	enum EPostRenameStatus
 	{
 		rsNone,
 		rsSkipped,
-		rsFailure,
+		rsNothing,
 		rsSuccess
 	};
 
@@ -350,7 +374,15 @@ public:
 		psManual
 	};
 
-	enum EUnpackStatus
+	enum EDirectUnpackStatus
+	{
+		nsNone,
+		nsRunning,
+		nsFailure,
+		nsSuccess
+	};
+
+	enum EPostUnpackStatus
 	{
 		usNone,
 		usSkipped,
@@ -485,12 +517,18 @@ public:
 	void BuildDestDirName();
 	CString BuildFinalDirName();
 	CompletedFileList* GetCompletedFiles() { return &m_completedFiles; }
-	ERenameStatus GetRenameStatus() { return m_renameStatus; }
-	void SetRenameStatus(ERenameStatus renameStatus) { m_renameStatus = renameStatus; }
+	void SetDirectRenameStatus(EDirectRenameStatus renameStatus) { m_directRenameStatus = renameStatus; }
+	EDirectRenameStatus GetDirectRenameStatus() { return m_directRenameStatus; }
+	EPostRenameStatus GetParRenameStatus() { return m_parRenameStatus; }
+	void SetParRenameStatus(EPostRenameStatus renameStatus) { m_parRenameStatus = renameStatus; }
+	EPostRenameStatus GetRarRenameStatus() { return m_rarRenameStatus; }
+	void SetRarRenameStatus(EPostRenameStatus renameStatus) { m_rarRenameStatus = renameStatus; }
 	EParStatus GetParStatus() { return m_parStatus; }
 	void SetParStatus(EParStatus parStatus) { m_parStatus = parStatus; }
-	EUnpackStatus GetUnpackStatus() { return m_unpackStatus; }
-	void SetUnpackStatus(EUnpackStatus unpackStatus) { m_unpackStatus = unpackStatus; }
+	EDirectUnpackStatus GetDirectUnpackStatus() { return m_directUnpackStatus; }
+	void SetDirectUnpackStatus(EDirectUnpackStatus directUnpackStatus) { m_directUnpackStatus = directUnpackStatus; }
+	EPostUnpackStatus GetUnpackStatus() { return m_unpackStatus; }
+	void SetUnpackStatus(EPostUnpackStatus unpackStatus) { m_unpackStatus = unpackStatus; }
 	ECleanupStatus GetCleanupStatus() { return m_cleanupStatus; }
 	void SetCleanupStatus(ECleanupStatus cleanupStatus) { m_cleanupStatus = cleanupStatus; }
 	EMoveStatus GetMoveStatus() { return m_moveStatus; }
@@ -554,6 +592,8 @@ public:
 	void SetUnpackSec(int unpackSec) { m_unpackSec = unpackSec; }
 	time_t GetDownloadStartTime() { return m_downloadStartTime; }
 	void SetDownloadStartTime(time_t downloadStartTime) { m_downloadStartTime = downloadStartTime; }
+	bool GetChanged() { return m_changed; }
+	void SetChanged(bool changed) { m_changed = changed; }
 	void SetReprocess(bool reprocess) { m_reprocess = reprocess; }
 	bool GetReprocess() { return m_reprocess; }
 	time_t GetQueueScriptTime() { return m_queueScriptTime; }
@@ -575,9 +615,18 @@ public:
 	void SetMessageCount(int messageCount) { m_messageCount = messageCount; }
 	int GetCachedMessageCount() { return m_cachedMessageCount; }
 	GuardedMessageList GuardCachedMessages() { return GuardedMessageList(&m_messages, &m_logMutex); }
+	bool GetAllFirst() { return m_allFirst; }
+	void SetAllFirst(bool allFirst) { m_allFirst = allFirst; }
+	bool GetWaitingPar() { return m_waitingPar; }
+	void SetWaitingPar(bool waitingPar) { m_waitingPar = waitingPar; }
+	bool GetLoadingPar() { return m_loadingPar; }
+	void SetLoadingPar(bool loadingPar) { m_loadingPar = loadingPar; }
+	Thread* GetUnpackThread() { return m_unpackThread; }
+	void SetUnpackThread(Thread* unpackThread) { m_unpackThread = unpackThread; }
 	void UpdateCurrentStats();
 	void UpdateCompletedStats(FileInfo* fileInfo);
 	void UpdateDeletedStats(FileInfo* fileInfo);
+	bool IsDownloadCompleted(bool ignorePausedPars);
 
 	static const int FORCE_PRIORITY = 900;
 
@@ -616,9 +665,12 @@ private:
 	time_t m_maxTime = 0;
 	int m_priority = 0;
 	CompletedFileList m_completedFiles;
-	ERenameStatus m_renameStatus = rsNone;
+	EDirectRenameStatus m_directRenameStatus = tsNone;
+	EPostRenameStatus m_parRenameStatus = rsNone;
+	EPostRenameStatus m_rarRenameStatus = rsNone;
 	EParStatus m_parStatus = psNone;
-	EUnpackStatus m_unpackStatus = usNone;
+	EDirectUnpackStatus m_directUnpackStatus = nsNone;
+	EPostUnpackStatus m_unpackStatus = usNone;
 	ECleanupStatus m_cleanupStatus = csNone;
 	EMoveStatus m_moveStatus = msNone;
 	EDeleteStatus m_deleteStatus = dsNone;
@@ -652,17 +704,23 @@ private:
 	std::unique_ptr<PostInfo> m_postInfo;
 	int64 m_downloadedSize = 0;
 	time_t m_downloadStartTime = 0;
+	int m_downloadStartSec = 0;
 	int m_downloadSec = 0;
 	int m_postTotalSec = 0;
 	int m_parSec = 0;
 	int m_repairSec = 0;
 	int m_unpackSec = 0;
 	bool m_reprocess = false;
+	bool m_changed = false;
 	time_t m_queueScriptTime = 0;
 	bool m_parFull = false;
 	int m_messageCount = 0;
 	int m_cachedMessageCount = 0;
 	int m_feedId = 0;
+	bool m_allFirst = false;
+	bool m_waitingPar = false;
+	bool m_loadingPar = false;
+	Thread* m_unpackThread = nullptr;
 
 	static int m_idGen;
 	static int m_idMax;
@@ -685,14 +743,17 @@ public:
 		ptVerifyingSources,
 		ptRepairing,
 		ptVerifyingRepaired,
-		ptRenaming,
+		ptParRenaming,
+		ptRarRenaming,
 		ptUnpacking,
+		ptCleaningUp,
 		ptMoving,
 		ptExecutingScript,
 		ptFinished
 	};
 
 	typedef std::vector<CString> ParredFiles;
+	typedef std::vector<CString> ExtractedArchives;
 
 	NzbInfo* GetNzbInfo() { return m_nzbInfo; }
 	void SetNzbInfo(NzbInfo* nzbInfo) { m_nzbInfo = nzbInfo; }
@@ -726,9 +787,12 @@ public:
 	void SetPassListTried(bool passListTried) { m_passListTried = passListTried; }
 	int GetLastUnpackStatus() { return m_lastUnpackStatus; }
 	void SetLastUnpackStatus(int unpackStatus) { m_lastUnpackStatus = unpackStatus; }
+	bool GetNeedParCheck() { return m_needParCheck; }
+	void SetNeedParCheck(bool needParCheck) { m_needParCheck = needParCheck; }
 	Thread* GetPostThread() { return m_postThread; }
 	void SetPostThread(Thread* postThread) { m_postThread = postThread; }
 	ParredFiles* GetParredFiles() { return &m_parredFiles; }
+	ExtractedArchives* GetExtractedArchives() { return &m_extractedArchives; }
 
 private:
 	NzbInfo* m_nzbInfo = nullptr;
@@ -741,6 +805,7 @@ private:
 	bool m_unpackTried = false;
 	bool m_passListTried = false;
 	int m_lastUnpackStatus = 0;
+	bool m_needParCheck = false;
 	EStage m_stage = ptQueued;
 	CString m_progressLabel = "";
 	int m_fileProgress = 0;
@@ -749,6 +814,7 @@ private:
 	time_t m_stageTime = 0;
 	Thread* m_postThread = nullptr;
 	ParredFiles m_parredFiles;
+	ExtractedArchives m_extractedArchives;
 };
 
 typedef std::vector<int> IdList;
@@ -842,6 +908,7 @@ public:
 		eaNzbFound,
 		eaNzbAdded,
 		eaNzbDeleted,
+		eaNzbNamed,
 		eaFileCompleted,
 		eaFileDeleted,
 		eaUrlCompleted
@@ -864,12 +931,14 @@ public:
 		eaFileResume, // resume (unpause) files
 		eaFileDelete, // delete files
 		eaFilePauseAllPars, // pause only (all) pars (does not affect other files)
-		eaFilePauseExtraPars, // pause only (almost all) pars, except main par-file (does not affect other files)
+		eaFilePauseExtraPars, // pause (almost all) pars, except main par-file (does not affect other files)
 		eaFileReorder, // set file order
 		eaFileSplit, // split - create new group from selected files
-		eaGroupMoveOffset, // move group to m_iOffset relative to the current position in download-queue
+		eaGroupMoveOffset, // move group to offset relative to the current position in download-queue
 		eaGroupMoveTop, // move group to the top of download-queue
 		eaGroupMoveBottom, // move group to the bottom of download-queue
+		eaGroupMoveBefore, // move group to a certain position
+		eaGroupMoveAfter, // move group to a certain position
 		eaGroupPause, // pause group
 		eaGroupResume, // resume (unpause) group
 		eaGroupDelete, // delete group and put to history, delete already downloaded files
@@ -888,6 +957,7 @@ public:
 		eaGroupSetDupeScore, // set duplicate score
 		eaGroupSetDupeMode, // set duplicate mode
 		eaGroupSort, // sort groups
+		eaGroupSortFiles, // sort files for optimal download order
 		eaPostDelete, // cancel post-processing
 		eaHistoryDelete, // hide history-item
 		eaHistoryFinalDelete, // delete history-item
@@ -918,8 +988,8 @@ public:
 	static GuardedDownloadQueue Guard() { return GuardedDownloadQueue(g_DownloadQueue, &g_DownloadQueue->m_lockMutex); }
 	NzbList* GetQueue() { return &m_queue; }
 	HistoryList* GetHistory() { return &m_history; }
-	virtual bool EditEntry(int ID, EEditAction action, int offset, const char* text) = 0;
-	virtual bool EditList(IdList* idList, NameList* nameList, EMatchMode matchMode, EEditAction action, int offset, const char* text) = 0;
+	virtual bool EditEntry(int ID, EEditAction action, const char* args) = 0;
+	virtual bool EditList(IdList* idList, NameList* nameList, EMatchMode matchMode, EEditAction action, const char* args) = 0;
 	virtual void HistoryChanged() = 0;
 	virtual void Save() = 0;
 	void CalcRemainingSize(int64* remaining, int64* remainingForced);
