@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -130,8 +130,8 @@ void HistoryCoordinator::AddToHistory(DownloadQueue* downloadQueue, NzbInfo* nzb
 	for (FileInfo* fileInfo : nzbInfo->GetFileList())
 	{
 		nzbInfo->UpdateCompletedStats(fileInfo);
-		nzbInfo->GetCompletedFiles()->emplace_back(fileInfo->GetId(),
-			fileInfo->GetFilename(), CompletedFile::cfNone, 0);
+		nzbInfo->GetCompletedFiles()->emplace_back(fileInfo->GetId(), fileInfo->GetFilename(),
+			CompletedFile::cfNone, 0, fileInfo->GetParFile(), fileInfo->GetHash16k(), fileInfo->GetParSetId());
 	}
 
 	// Cleaning up parked files if par-check was successful or unpack was successful or
@@ -169,7 +169,7 @@ void HistoryCoordinator::AddToHistory(DownloadQueue* downloadQueue, NzbInfo* nzb
 			(completedFile.GetStatus() == CompletedFile::cfPartial &&
 			 &completedFile == &*nzbInfo->GetCompletedFiles()->rbegin()))
 		{
-			nzbInfo->PrintMessage(Message::mkDetail, "Parking file %s", completedFile.GetFileName());
+			nzbInfo->PrintMessage(Message::mkDetail, "Parking file %s", completedFile.GetFilename());
 			nzbInfo->SetParkedFileCount(nzbInfo->GetParkedFileCount() + 1);
 		}
 	}
@@ -178,6 +178,11 @@ void HistoryCoordinator::AddToHistory(DownloadQueue* downloadQueue, NzbInfo* nzb
 	nzbInfo->SetRemainingParCount(0);
 	nzbInfo->SetParking(false);
 
+	if (nzbInfo->GetDirectRenameStatus() == NzbInfo::tsRunning)
+	{
+		nzbInfo->SetDirectRenameStatus(NzbInfo::tsFailure);
+	}
+
 	nzbInfo->PrintMessage(Message::mkInfo, "Collection %s added to history", nzbInfo->GetName());
 }
 
@@ -232,7 +237,8 @@ void HistoryCoordinator::PrepareEdit(DownloadQueue* downloadQueue, IdList* idLis
 	}
 }
 
-bool HistoryCoordinator::EditList(DownloadQueue* downloadQueue, IdList* idList, DownloadQueue::EEditAction action, int offset, const char* text)
+bool HistoryCoordinator::EditList(DownloadQueue* downloadQueue, IdList* idList,
+	DownloadQueue::EEditAction action, const char* args)
 {
 	bool ok = false;
 	PrepareEdit(downloadQueue, idList, action);
@@ -270,22 +276,22 @@ bool HistoryCoordinator::EditList(DownloadQueue* downloadQueue, IdList* idList,
 						break;
 
 					case DownloadQueue::eaHistorySetParameter:
-						ok = HistorySetParameter(historyInfo, text);
+						ok = HistorySetParameter(historyInfo, args);
 						break;
 
  					case DownloadQueue::eaHistorySetCategory:
-						ok = HistorySetCategory(historyInfo, text);
+						ok = HistorySetCategory(historyInfo, args);
 						break;
 
  					case DownloadQueue::eaHistorySetName:
-						ok = HistorySetName(historyInfo, text);
+						ok = HistorySetName(historyInfo, args);
 						break;
 
 					case DownloadQueue::eaHistorySetDupeKey:
 					case DownloadQueue::eaHistorySetDupeScore:
 					case DownloadQueue::eaHistorySetDupeMode:
 					case DownloadQueue::eaHistorySetDupeBackup:
-						HistorySetDupeParam(historyInfo, action, text);
+						HistorySetDupeParam(historyInfo, action, args);
 						break;
 
 					case DownloadQueue::eaHistoryMarkBad:
@@ -378,8 +384,10 @@ void HistoryCoordinator::MoveToQueue(DownloadQueue* downloadQueue, HistoryList::
 	if (!nzbInfo->GetUnpackCleanedUpDisk())
 	{
 		nzbInfo->SetUnpackStatus(NzbInfo::usNone);
+		nzbInfo->SetDirectUnpackStatus(NzbInfo::nsNone);
 		nzbInfo->SetCleanupStatus(NzbInfo::csNone);
-		nzbInfo->SetRenameStatus(NzbInfo::rsNone);
+		nzbInfo->SetParRenameStatus(NzbInfo::rsNone);
+		nzbInfo->SetRarRenameStatus(NzbInfo::rsNone);
 		nzbInfo->SetPostTotalSec(nzbInfo->GetPostTotalSec() - nzbInfo->GetUnpackSec());
 		nzbInfo->SetUnpackSec(0);
 
@@ -489,7 +497,10 @@ void HistoryCoordinator::HistoryRedownload(DownloadQueue* downloadQueue, History
 	nzbInfo->SetMoveStatus(NzbInfo::msNone);
 	nzbInfo->SetUnpackCleanedUpDisk(false);
 	nzbInfo->SetParStatus(NzbInfo::psNone);
-	nzbInfo->SetRenameStatus(NzbInfo::rsNone);
+	nzbInfo->SetParRenameStatus(NzbInfo::rsNone);
+	nzbInfo->SetRarRenameStatus(NzbInfo::rsNone);
+	nzbInfo->SetDirectRenameStatus(NzbInfo::tsNone);
+	nzbInfo->SetDirectUnpackStatus(NzbInfo::nsNone);
 	nzbInfo->SetDownloadedSize(0);
 	nzbInfo->SetDownloadSec(0);
 	nzbInfo->SetPostTotalSec(0);
@@ -497,6 +508,9 @@ void HistoryCoordinator::HistoryRedownload(DownloadQueue* downloadQueue, History
 	nzbInfo->SetRepairSec(0);
 	nzbInfo->SetUnpackSec(0);
 	nzbInfo->SetExtraParBlocks(0);
+	nzbInfo->SetAllFirst(false);
+	nzbInfo->SetWaitingPar(false);
+	nzbInfo->SetLoadingPar(false);
 	nzbInfo->GetCompletedFiles()->clear();
 	nzbInfo->GetServerStats()->clear();
 	nzbInfo->GetCurrentServerStats()->clear();
@@ -584,7 +598,7 @@ void HistoryCoordinator::HistoryRetry(DownloadQueue* downloadQueue, HistoryList:
 				  g_DiskState->LoadFileState(fileInfo.get(), g_ServerPool->GetServers(), true) &&
 				  (resetFailed || fileInfo->GetRemainingSize() > 0))))
 			{
-				fileInfo->SetFilename(completedFile.GetFileName());
+				fileInfo->SetFilename(completedFile.GetFilename());
 				fileInfo->SetNzbInfo(nzbInfo);
 
 				BString<1024> outputFilename("%s%c%s", nzbInfo->GetDestDir(), PATH_SEPARATOR, fileInfo->GetFilename());
@@ -631,7 +645,7 @@ void HistoryCoordinator::HistoryRetry(DownloadQueue* downloadQueue, HistoryList:
 
 	if (g_Options->GetParCheck() != Options::pcForce)
 	{
-		downloadQueue->EditEntry(nzbInfo->GetId(), DownloadQueue::eaGroupPauseExtraPars, 0, nullptr);
+		downloadQueue->EditEntry(nzbInfo->GetId(), DownloadQueue::eaGroupPauseExtraPars, nullptr);
 	}
 }
 
