@@ -2,7 +2,7 @@
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@users.sourceforge.net>
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,16 +32,16 @@
 #include "StatMeter.h"
 
 bool QueueCoordinator::CoordinatorDownloadQueue::EditEntry(
-	int ID, EEditAction action, int offset, const char* text)
+	int ID, EEditAction action, const char* args)
 {
-	return m_owner->m_queueEditor.EditEntry(&m_owner->m_downloadQueue, ID, action, offset, text);
+	return m_owner->m_queueEditor.EditEntry(&m_owner->m_downloadQueue, ID, action, args);
 }
 
 bool QueueCoordinator::CoordinatorDownloadQueue::EditList(
-	IdList* idList, NameList* nameList, EMatchMode matchMode, EEditAction action, int offset, const char* text)
+	IdList* idList, NameList* nameList, EMatchMode matchMode, EEditAction action, const char* args)
 {
 	m_massEdit = true;
-	bool ret = m_owner->m_queueEditor.EditList(&m_owner->m_downloadQueue, idList, nameList, matchMode, action, offset, text);
+	bool ret = m_owner->m_queueEditor.EditList(&m_owner->m_downloadQueue, idList, nameList, matchMode, action, args);
 	m_massEdit = false;
 	if (m_wantSave)
 	{
@@ -63,6 +63,11 @@ void QueueCoordinator::CoordinatorDownloadQueue::Save()
 		g_DiskState->SaveDownloadQueue(this, m_historyChanged);
 	}
 
+	for (NzbInfo* nzbInfo : GetQueue())
+	{
+		nzbInfo->SetChanged(false);
+	}
+
 	m_wantSave = false;
 	m_historyChanged = false;
 }
@@ -71,7 +76,6 @@ QueueCoordinator::QueueCoordinator()
 {
 	debug("Creating QueueCoordinator");
 
-	m_downloadQueue.m_owner = this;
 	CoordinatorDownloadQueue::Init(&m_downloadQueue);
 }
 
@@ -225,7 +229,7 @@ void QueueCoordinator::Run()
 			wasStandBy = standBy;
 			if (standBy)
 			{
-				SavePartialState();
+				SaveAllPartialState();
 			}
 		}
 
@@ -248,7 +252,7 @@ void QueueCoordinator::Run()
 			ResetHangingDownloads();
 			if (!standBy)
 			{
-				SavePartialState();
+				SaveAllPartialState();
 			}
 			resetCounter = 0;
 			g_StatMeter->IntervalCheck();
@@ -256,23 +260,31 @@ void QueueCoordinator::Run()
 		}
 	}
 
+	WaitJobs();
+	SaveAllPartialState();
+
+	debug("Exiting QueueCoordinator-loop");
+}
+
+void QueueCoordinator::WaitJobs()
+{
 	// waiting for downloads
 	debug("QueueCoordinator: waiting for Downloads to complete");
-	bool completed = false;
-	while (!completed)
+
+	while (true)
 	{
 		{
 			GuardedDownloadQueue guard = DownloadQueue::Guard();
-			completed = m_activeDownloads.size() == 0;
+			if (m_activeDownloads.empty())
+			{
+				break;
+			}
 		}
 		usleep(100 * 1000);
 		ResetHangingDownloads();
 	}
+
 	debug("QueueCoordinator: Downloads are completed");
-
-	SavePartialState();
-
-	debug("Exiting QueueCoordinator-loop");
 }
 
 /*
@@ -371,7 +383,10 @@ NzbInfo* QueueCoordinator::AddNzbFileToQueue(std::unique_ptr<NzbInfo> nzbInfo, N
 	{
 		// in a case if none of listeners did already delete the temporary object - we do it ourselves
 		downloadQueue->GetQueue()->Remove(addedNzb);
-		addedNzb = nullptr;
+		if (!downloadQueue->GetHistory()->Find(addedNzb->GetId()))
+		{
+			addedNzb = nullptr;
+		}
 	}
 
 	downloadQueue->Save();
@@ -489,6 +504,14 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* downloadQueue, FileInfo* &f
 			break;
 		}
 
+		if (g_Options->GetDirectRename() &&
+			fileInfo->GetNzbInfo()->GetDirectRenameStatus() <= NzbInfo::tsRunning &&
+			!fileInfo->GetNzbInfo()->GetAllFirst() &&
+			GetNextFirstArticle(fileInfo->GetNzbInfo(), fileInfo, articleInfo))
+		{
+			return true;
+		}
+
 		if (fileInfo->GetArticles()->empty() && g_Options->GetSaveQueue() && g_Options->GetServerMode())
 		{
 			g_DiskState->LoadArticles(fileInfo);
@@ -501,8 +524,7 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* downloadQueue, FileInfo* &f
 			if (article->GetStatus() == ArticleInfo::aiUndefined)
 			{
 				articleInfo = article;
-				ok = true;
-				break;
+				return true;
 			}
 		}
 
@@ -514,7 +536,39 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* downloadQueue, FileInfo* &f
 		}
 	}
 
-	return ok;
+	return false;
+}
+
+bool QueueCoordinator::GetNextFirstArticle(NzbInfo* nzbInfo, FileInfo* &fileInfo, ArticleInfo* &articleInfo)
+{
+	// find a file not renamed yet
+	for (FileInfo* fileInfo1 : nzbInfo->GetFileList())
+	{
+		if (!fileInfo1->GetFilenameConfirmed())
+		{
+			if (fileInfo1->GetArticles()->empty() && g_Options->GetSaveQueue() && g_Options->GetServerMode())
+			{
+				g_DiskState->LoadArticles(fileInfo1);
+				LoadPartialState(fileInfo1);
+			}
+			if (!fileInfo1->GetArticles()->empty())
+			{
+				ArticleInfo* article = fileInfo1->GetArticles()->at(0).get();
+				if (article->GetStatus() == ArticleInfo::aiUndefined)
+				{
+					fileInfo = fileInfo1;
+					articleInfo = article;
+					nzbInfo->SetDirectRenameStatus(NzbInfo::tsRunning);
+					return true;
+				}
+			}
+		}
+	}
+
+	// no more files for renaming remained
+	nzbInfo->SetAllFirst(true);
+
+	return false;
 }
 
 void QueueCoordinator::StartArticleDownload(FileInfo* fileInfo, ArticleInfo* articleInfo, NntpConnection* connection)
@@ -528,7 +582,12 @@ void QueueCoordinator::StartArticleDownload(FileInfo* fileInfo, ArticleInfo* art
 	articleDownloader->SetArticleInfo(articleInfo);
 	articleDownloader->SetConnection(connection);
 
-	BString<1024> infoName("%s%c%s [%i/%i]", fileInfo->GetNzbInfo()->GetName(), (int)PATH_SEPARATOR, fileInfo->GetFilename(), articleInfo->GetPartNumber(), (int)fileInfo->GetArticles()->size());
+	if (articleInfo->GetPartNumber() == 1 && g_Options->GetDirectRename() && g_Options->GetDecode())
+	{
+		articleDownloader->SetContentAnalyzer(m_directRenamer.MakeArticleContentAnalyzer());
+	}
+
+	BString<1024> infoName("%s%c%s [%i/%i]", fileInfo->GetNzbInfo()->GetName(), PATH_SEPARATOR, fileInfo->GetFilename(), articleInfo->GetPartNumber(), (int)fileInfo->GetArticles()->size());
 	articleDownloader->SetInfoName(infoName);
 
 	articleInfo->SetStatus(ArticleInfo::aiRunning);
@@ -557,12 +616,14 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* articleDownloader)
 	debug("Article downloaded");
 
 	FileInfo* fileInfo = articleDownloader->GetFileInfo();
-	NzbInfo* nzbInfo = fileInfo->GetNzbInfo();
-	ArticleInfo* articleInfo = articleDownloader->GetArticleInfo();
-	bool retry = false;
-	bool fileCompleted = false;
+	bool completeFileParts = false;
 
 	{
+		NzbInfo* nzbInfo = fileInfo->GetNzbInfo();
+		ArticleInfo* articleInfo = articleDownloader->GetArticleInfo();
+		bool retry = false;
+		bool fileCompleted = false;
+
 		GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
 
 		if (articleDownloader->GetStatus() == ArticleDownloader::adFinished)
@@ -587,6 +648,10 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* articleDownloader)
 		{
 			articleInfo->SetStatus(ArticleInfo::aiUndefined);
 			retry = true;
+			if (articleInfo->GetPartNumber() == 1)
+			{
+				nzbInfo->SetAllFirst(false);
+			}
 		}
 
 		if (!retry)
@@ -608,13 +673,22 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* articleDownloader)
 			articleDownloader->GetStatus() == ArticleDownloader::adFinished &&
 			articleDownloader->GetArticleFilename())
 		{
-			fileInfo->SetFilename(articleDownloader->GetArticleFilename());
-			fileInfo->MakeValidFilename();
+			// in "FileNaming=auto"-mode prefer filename from nzb-file to filename read from article
+			// if the name from article seems to be obfuscated
+			bool useFilenameFromArticle = g_Options->GetFileNaming() == Options::nfArticle ||
+				(g_Options->GetFileNaming() == Options::nfAuto &&
+				 !Util::AlphaNum(articleDownloader->GetArticleFilename()) &&
+				 !nzbInfo->GetManyDupeFiles());
+			if (useFilenameFromArticle)
+			{
+				fileInfo->SetFilename(articleDownloader->GetArticleFilename());
+				fileInfo->MakeValidFilename();
+			}
 			fileInfo->SetFilenameConfirmed(true);
 			if (g_Options->GetDupeCheck() &&
 				nzbInfo->GetDupeMode() != dmForce &&
 				!nzbInfo->GetManyDupeFiles() &&
-				FileSystem::FileExists(nzbInfo->GetDestDir(), fileInfo->GetFilename()))
+				FileSystem::FileExists(BString<1024>("%s%c%s", nzbInfo->GetDestDir(), PATH_SEPARATOR, fileInfo->GetFilename())))
 			{
 				warn("File \"%s\" seems to be duplicate, cancelling download and deleting file from queue", fileInfo->GetFilename());
 				fileCompleted = false;
@@ -623,6 +697,11 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* articleDownloader)
 			}
 		}
 
+		if (articleDownloader->GetContentAnalyzer() && articleDownloader->GetStatus() == ArticleDownloader::adFinished)
+		{
+			m_directRenamer.ArticleDownloaded(downloadQueue, fileInfo, articleInfo, articleDownloader->GetContentAnalyzer());
+		}
+
 		nzbInfo->SetDownloadedSize(nzbInfo->GetDownloadedSize() + articleDownloader->GetDownloadedSize());
 
 		CheckHealth(downloadQueue, fileInfo);
@@ -631,35 +710,44 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* articleDownloader)
 		{
 			fileCompleted = true;
 		}
+
+		completeFileParts = fileCompleted && (!fileInfo->GetDeleted() || nzbInfo->GetParking());
+
+		if (!completeFileParts)
+		{
+			DeleteDownloader(downloadQueue, articleDownloader, false);
+		}
 	}
 
-	bool deleteFileObj = false;
-
-	if (fileCompleted && (!fileInfo->GetDeleted() || nzbInfo->GetParking()))
+	if (completeFileParts)
 	{
 		// all jobs done
 		articleDownloader->CompleteFileParts();
 		fileInfo->SetPartialChanged(false);
-		deleteFileObj = true;
-	}
 
-	{
 		GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
+		DeleteDownloader(downloadQueue, articleDownloader, true);
+	}
+}
 
-		bool hasOtherDownloaders = fileInfo->GetActiveDownloads() > 1;
-		deleteFileObj |= fileInfo->GetDeleted() && !hasOtherDownloaders;
+void QueueCoordinator::DeleteDownloader(DownloadQueue* downloadQueue,
+	ArticleDownloader* articleDownloader, bool fileCompleted)
+{
+	FileInfo* fileInfo = articleDownloader->GetFileInfo();
+	NzbInfo* nzbInfo = fileInfo->GetNzbInfo();
+	bool hasOtherDownloaders = fileInfo->GetActiveDownloads() > 1;
+	bool deleteFileObj = fileCompleted || (fileInfo->GetDeleted() && !hasOtherDownloaders);
 
-		// remove downloader from downloader list
-		m_activeDownloads.erase(std::find(m_activeDownloads.begin(), m_activeDownloads.end(), articleDownloader));
+	// remove downloader from downloader list
+	m_activeDownloads.erase(std::find(m_activeDownloads.begin(), m_activeDownloads.end(), articleDownloader));
 
-		fileInfo->SetActiveDownloads(fileInfo->GetActiveDownloads() - 1);
-		nzbInfo->SetActiveDownloads(nzbInfo->GetActiveDownloads() - 1);
+	fileInfo->SetActiveDownloads(fileInfo->GetActiveDownloads() - 1);
+	nzbInfo->SetActiveDownloads(nzbInfo->GetActiveDownloads() - 1);
 
-		if (deleteFileObj)
-		{
-			DeleteFileInfo(downloadQueue, fileInfo, fileCompleted);
-			downloadQueue->Save();
-		}
+	if (deleteFileObj)
+	{
+		DeleteFileInfo(downloadQueue, fileInfo, fileCompleted);
+		downloadQueue->Save();
 	}
 }
 
@@ -712,7 +800,19 @@ void QueueCoordinator::DeleteFileInfo(DownloadQueue* downloadQueue, FileInfo* fi
 			completed && fileInfo->GetOutputFilename() ?
 			FileSystem::BaseFileName(fileInfo->GetOutputFilename()) : fileInfo->GetFilename(),
 			fileStatus,
-			fileStatus == CompletedFile::cfSuccess ? fileInfo->GetCrc() : 0);
+			fileStatus == CompletedFile::cfSuccess ? fileInfo->GetCrc() : 0,
+			fileInfo->GetParFile(), fileInfo->GetHash16k(), fileInfo->GetParSetId());
+	}
+
+	if (g_Options->GetDirectRename())
+	{
+		m_directRenamer.FileDownloaded(downloadQueue, fileInfo);
+	}
+
+	if (nzbInfo->GetDirectRenameStatus() == NzbInfo::tsRunning &&
+		!nzbInfo->GetDeleting() && nzbInfo->IsDownloadCompleted(true))
+	{
+		DiscardDirectRename(downloadQueue, nzbInfo);
 	}
 
 	std::unique_ptr<FileInfo> srcFileInfo = nzbInfo->GetFileList()->Remove(fileInfo);
@@ -745,30 +845,45 @@ void QueueCoordinator::DiscardTempFiles(FileInfo* fileInfo)
 	}
 }
 
-void QueueCoordinator::SavePartialState()
+void QueueCoordinator::SaveAllPartialState()
 {
-	if (!(g_Options->GetServerMode() && g_Options->GetSaveQueue() && g_Options->GetContinuePartial()))
+	if (!(g_Options->GetServerMode() && g_Options->GetSaveQueue()))
 	{
 		return;
 	}
 
+	bool hasUnsavedData = false;
 	GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
 	for (NzbInfo* nzbInfo : downloadQueue->GetQueue())
 	{
-		for (FileInfo* fileInfo : nzbInfo->GetFileList())
+		if (g_Options->GetContinuePartial())
 		{
-			if (fileInfo->GetPartialChanged())
+			for (FileInfo* fileInfo : nzbInfo->GetFileList())
 			{
-				debug("Saving partial state for %s", fileInfo->GetFilename());
-				if (fileInfo->GetPartialState() == FileInfo::psCompleted)
-				{
-					g_DiskState->DiscardFile(fileInfo->GetId(), false, false, true);
-				}
-				g_DiskState->SaveFileState(fileInfo, false);
-				fileInfo->SetPartialChanged(false);
-				fileInfo->SetPartialState(FileInfo::psPartial);
+				SavePartialState(fileInfo);
 			}
 		}
+		hasUnsavedData |= nzbInfo->GetChanged();
+	}
+
+	if (hasUnsavedData)
+	{
+		downloadQueue->Save();
+	}
+}
+
+void QueueCoordinator::SavePartialState(FileInfo* fileInfo)
+{
+	if (fileInfo->GetPartialChanged())
+	{
+		debug("Saving partial state for %s", fileInfo->GetFilename());
+		if (fileInfo->GetPartialState() == FileInfo::psCompleted)
+		{
+			g_DiskState->DiscardFile(fileInfo->GetId(), false, false, true);
+		}
+		g_DiskState->SaveFileState(fileInfo, false);
+		fileInfo->SetPartialChanged(false);
+		fileInfo->SetPartialState(FileInfo::psPartial);
 	}
 }
 
@@ -808,7 +923,7 @@ void QueueCoordinator::CheckHealth(DownloadQueue* downloadQueue, FileInfo* fileI
 		warn("Pausing %s due to health %.1f%% below critical %.1f%%", fileInfo->GetNzbInfo()->GetName(),
 			fileInfo->GetNzbInfo()->CalcHealth() / 10.0, fileInfo->GetNzbInfo()->CalcCriticalHealth(true) / 10.0);
 		fileInfo->GetNzbInfo()->SetHealthPaused(true);
-		downloadQueue->EditEntry(fileInfo->GetNzbInfo()->GetId(), DownloadQueue::eaGroupPause, 0, nullptr);
+		downloadQueue->EditEntry(fileInfo->GetNzbInfo()->GetId(), DownloadQueue::eaGroupPause, nullptr);
 	}
 	else if (g_Options->GetHealthCheck() == Options::hcDelete ||
 		g_Options->GetHealthCheck() == Options::hcPark)
@@ -820,7 +935,7 @@ void QueueCoordinator::CheckHealth(DownloadQueue* downloadQueue, FileInfo* fileI
 		fileInfo->GetNzbInfo()->SetDeleteStatus(NzbInfo::dsHealth);
 		downloadQueue->EditEntry(fileInfo->GetNzbInfo()->GetId(),
 			g_Options->GetHealthCheck() == Options::hcPark ? DownloadQueue::eaGroupParkDelete : DownloadQueue::eaGroupDelete,
-			0, nullptr);
+			nullptr);
 	}
 }
 
@@ -1172,3 +1287,110 @@ bool QueueCoordinator::SplitQueueEntries(DownloadQueue* downloadQueue, RawFileLi
 
 	return true;
 }
+
+void QueueCoordinator::DirectRenameCompleted(DownloadQueue* downloadQueue, NzbInfo* nzbInfo)
+{
+	for (FileInfo* fileInfo : nzbInfo->GetFileList())
+	{
+		if (g_Options->GetSaveQueue() && g_Options->GetServerMode() && !fileInfo->GetArticles()->empty())
+		{
+			// save new file name into disk state file
+			g_DiskState->SaveFile(fileInfo);
+		}
+	}
+
+	DiscardDirectRename(downloadQueue, nzbInfo);
+
+	nzbInfo->SetDirectRenameStatus(NzbInfo::tsSuccess);
+
+	if (g_Options->GetParCheck() != Options::pcForce)
+	{
+		downloadQueue->EditEntry(nzbInfo->GetId(), DownloadQueue::eaGroupResume, nullptr);
+		downloadQueue->EditEntry(nzbInfo->GetId(), DownloadQueue::eaGroupPauseAllPars, nullptr);
+	}
+
+	if (g_Options->GetReorderFiles())
+	{
+		nzbInfo->PrintMessage(Message::mkInfo, "Reordering files for %s", nzbInfo->GetName());
+		downloadQueue->EditEntry(nzbInfo->GetId(), DownloadQueue::eaGroupSortFiles, nullptr);
+	}
+
+	downloadQueue->Save();
+
+	DownloadQueue::Aspect namedAspect = { DownloadQueue::eaNzbNamed, downloadQueue, nzbInfo, nullptr };
+	downloadQueue->Notify(&namedAspect);
+}
+
+void QueueCoordinator::DiscardDirectRename(DownloadQueue* downloadQueue, NzbInfo* nzbInfo)
+{
+	int64 discardedSize = 0;
+	int discardedCount = 0;
+
+	for (FileInfo* fileInfo : nzbInfo->GetFileList())
+	{
+		if (fileInfo->GetParFile() && fileInfo->GetCompletedArticles() == 1 && fileInfo->GetActiveDownloads() == 0)
+		{
+			// discard downloaded articles from partially downloaded par-files
+			discardedSize += fileInfo->GetSuccessSize();
+			discardedCount++;
+
+			nzbInfo->SetCurrentSuccessArticles(nzbInfo->GetCurrentSuccessArticles() - fileInfo->GetSuccessArticles());
+			nzbInfo->SetCurrentSuccessSize(nzbInfo->GetCurrentSuccessSize() - fileInfo->GetSuccessSize());
+			nzbInfo->SetParCurrentSuccessSize(nzbInfo->GetParCurrentSuccessSize() - fileInfo->GetSuccessSize());
+			fileInfo->SetSuccessSize(0);
+			fileInfo->SetSuccessArticles(0);
+
+			nzbInfo->SetCurrentFailedArticles(nzbInfo->GetCurrentFailedArticles() - fileInfo->GetFailedArticles());
+			nzbInfo->SetCurrentFailedSize(nzbInfo->GetCurrentFailedSize() - fileInfo->GetFailedSize());
+			nzbInfo->SetParCurrentFailedSize(nzbInfo->GetParCurrentFailedSize() - fileInfo->GetFailedSize());
+			fileInfo->SetFailedSize(0);
+			fileInfo->SetFailedArticles(0);
+
+			fileInfo->SetCompletedArticles(0);
+			fileInfo->SetRemainingSize(fileInfo->GetSize() - fileInfo->GetMissedSize());
+
+			// discard temporary files
+			DiscardTempFiles(fileInfo);
+			g_DiskState->DiscardFile(fileInfo->GetId(), false, true, false);
+
+			fileInfo->SetOutputFilename(nullptr);
+			fileInfo->SetOutputInitialized(false);
+			fileInfo->SetCachedArticles(0);
+			fileInfo->SetPartialChanged(false);
+			fileInfo->SetPartialState(FileInfo::psNone);
+
+			if (g_Options->GetSaveQueue() && g_Options->GetServerMode())
+			{
+				// free up memory used by articles if possible
+				fileInfo->GetArticles()->clear();
+			}
+			else
+			{
+				// reset article states if discarding isn't possible
+				for (ArticleInfo* articleInfo : fileInfo->GetArticles())
+				{
+					articleInfo->SetStatus(ArticleInfo::aiUndefined);
+					articleInfo->SetResultFilename(nullptr);
+					articleInfo->DiscardSegment();
+				}
+			}
+		}
+
+		if (g_Options->GetSaveQueue() && g_Options->GetServerMode() &&
+			!fileInfo->GetArticles()->empty() && g_Options->GetContinuePartial() &&
+			fileInfo->GetActiveDownloads() == 0 && fileInfo->GetCachedArticles() == 0)
+		{
+			// discard article infos to free up memory if possible
+			debug("Discarding article infos for %s/%s", nzbInfo->GetName(), fileInfo->GetFilename());
+			fileInfo->SetPartialChanged(true);
+			SavePartialState(fileInfo);
+			fileInfo->GetArticles()->clear();
+		}
+	}
+
+	if (discardedSize > 0)
+	{
+		nzbInfo->PrintMessage(Message::mkDetail, "Discarded %s from %i files used for direct renaming",
+			*Util::FormatSize(discardedSize), discardedCount);
+	}
+}
