@@ -1,8 +1,8 @@
 /*
- *  This file is part of nzbget
+ *  This file is part of nzbget. See <http://nzbget.net>.
  *
  *  Copyright (C) 2005 Bo Cordes Petersen <placebodk@users.sourceforge.net>
- *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,12 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *
- * $Revision$
- * $Date$
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -32,28 +27,27 @@
 
 class RemoteServer : public Thread
 {
-private:
-	bool				m_bTLS;
-	Connection*			m_pConnection;
-
 public:
-						RemoteServer(bool bTLS);
-						~RemoteServer();
-	virtual void		Run();
-	virtual void 		Stop();
+	RemoteServer(bool tls) : m_tls(tls) {}
+	virtual void Run();
+	virtual void Stop();
+
+private:
+	bool m_tls;
+	std::unique_ptr<Connection> m_connection;
 };
 
 class RequestProcessor : public Thread
 {
-private:
-	bool				m_bTLS;
-	Connection*			m_pConnection;
-
 public:
-						~RequestProcessor();
-	virtual void		Run();
-	void				SetTLS(bool bTLS) { m_bTLS = bTLS; }
-	void				SetConnection(Connection* pConnection) { m_pConnection = pConnection; }
+	~RequestProcessor();
+	virtual void Run();
+	void SetTls(bool tls) { m_tls = tls; }
+	void SetConnection(std::unique_ptr<Connection>&& connection) { m_connection = std::move(connection); }
+
+private:
+	bool m_tls;
+	std::unique_ptr<Connection> m_connection;
 };
 
 #endif
