@@ -90,7 +90,10 @@ void WebProcessor::Execute()
 
 	ParseUrl();
 
-	if (!CheckCredentials())
+	m_rpcRequest = XmlRpcProcessor::IsRpcRequest(m_url);
+	m_authorized = CheckCredentials();
+
+	if ((!g_Options->GetFormAuth() || m_rpcRequest) && !m_authorized)
 	{
 		SendAuthResponse();
 		return;
@@ -124,11 +127,12 @@ void WebProcessor::ParseHeaders()
 	{
 		if (char* pe = strrchr(p, '\r')) *pe = '\0';
 		debug("header=%s", p);
+
 		if (!strncasecmp(p, "Content-Length: ", 16))
 		{
 			m_contentLen = atoi(p + 16);
 		}
-		if (!strncasecmp(p, "Authorization: Basic ", 21))
+		else if (!strncasecmp(p, "Authorization: Basic ", 21) && Util::EmptyStr(m_authInfo))
 		{
 			char* authInfo64 = p + 21;
 			if (strlen(authInfo64) > sizeof(m_authInfo))
@@ -138,20 +142,39 @@ void WebProcessor::ParseHeaders()
 			}
 			m_authInfo[WebUtil::DecodeBase64(authInfo64, 0, m_authInfo)] = '\0';
 		}
-		if (!strncasecmp(p, "Accept-Encoding: ", 17))
+		else if (!strncasecmp(p, "X-Authorization: Basic ", 23))
+		{
+			char* authInfo64 = p + 23;
+			if (strlen(authInfo64) > sizeof(m_authInfo))
+			{
+				error("Invalid-request: auth-info too big");
+				return;
+			}
+			m_authInfo[WebUtil::DecodeBase64(authInfo64, 0, m_authInfo)] = '\0';
+		}
+		else if (!strncasecmp(p, "Accept-Encoding: ", 17))
 		{
 			m_gzip = strstr(p, "gzip");
 		}
-		if (!strncasecmp(p, "Origin: ", 8))
+		else if (!strncasecmp(p, "Origin: ", 8))
 		{
 			m_origin = p + 8;
 		}
-		if (!strncasecmp(p, "X-Auth-Token: ", 14))
+		else if (!strncasecmp(p, "Cookie: ", 8))
 		{
-			strncpy(m_authToken, p + 14, sizeof(m_authToken)-1);
-			m_authToken[sizeof(m_authToken)-1] = '\0';
+			debug("%s", p);
+			const char* tok = strstr(p, "Auth-Token=");
+			if (tok && tok[11] != ';' && tok[11] != '\0')
+			{
+				strncpy(m_authToken, tok + 11, sizeof(m_authToken)-1);
+				m_authToken[sizeof(m_authToken)-1] = '\0';
+			}
 		}
-		if (*p == '\0')
+		else if (!strncasecmp(p, "X-Forwarded-For: ", 17))
+		{
+			m_forwardedFor = p + 17;
+		}
+		else if (*p == '\0')
 		{
 			break;
 		}
@@ -159,7 +182,7 @@ void WebProcessor::ParseHeaders()
 
 	debug("URL=%s", *m_url);
 	debug("Authorization=%s", m_authInfo);
-	debug("X-Auth-Token=%s", m_authToken);
+	debug("Auth-Token=%s", m_authToken);
 }
 
 void WebProcessor::ParseUrl()
@@ -245,8 +268,10 @@ bool WebProcessor::CheckCredentials()
 		}
 		else
 		{
-			warn("Request received on port %i from %s, but username or password invalid (%s:%s)",
-				g_Options->GetControlPort(), m_connection->GetRemoteAddr(), m_authInfo, pw);
+			warn("Request received on port %i from %s%s, but username or password invalid (%s:%s)",
+				g_Options->GetControlPort(), m_connection->GetRemoteAddr(),
+				!m_forwardedFor.Empty() ? (char*)BString<1024>(" (forwarded for: %s)", *m_forwardedFor) : "",
+				m_authInfo, pw);
 			return false;
 		}
 	}
@@ -281,7 +306,7 @@ void WebProcessor::Dispatch()
 		return;
 	}
 
-	if (XmlRpcProcessor::IsRpcRequest(m_url))
+	if (m_rpcRequest)
 	{
 		XmlRpcProcessor processor;
 		processor.SetRequest(m_request);
@@ -333,12 +358,15 @@ void WebProcessor::SendAuthResponse()
 {
 	const char* AUTH_RESPONSE_HEADER =
 		"HTTP/1.0 401 Unauthorized\r\n"
-		"WWW-Authenticate: Basic realm=\"NZBGet\"\r\n"
+		"%s"
 		"Connection: close\r\n"
 		"Content-Type: text/plain\r\n"
 		"Server: nzbget-%s\r\n"
 		"\r\n";
-	BString<1024> responseHeader(AUTH_RESPONSE_HEADER, Util::VersionRevision());
+
+	BString<1024> responseHeader(AUTH_RESPONSE_HEADER,
+		g_Options->GetFormAuth() ? "" : "WWW-Authenticate: Basic realm=\"NZBGet\"\r\n",
+		Util::VersionRevision());
 
 	// Send the response answer
 	debug("ResponseHeader=%s", *responseHeader);
@@ -417,7 +445,8 @@ void WebProcessor::SendBodyResponse(const char* body, int bodyLen, const char* c
 		"Access-Control-Allow-Credentials: true\r\n"
 		"Access-Control-Max-Age: 86400\r\n"
 		"Access-Control-Allow-Headers: Content-Type, Authorization\r\n"
-		"X-Auth-Token: %s\r\n"
+		"Set-Cookie: Auth-Type=%s\r\n"
+		"Set-Cookie: Auth-Token=%s\r\n"
 		"Content-Length: %i\r\n"
 		"%s"					// Content-Type: xxx
 		"%s"					// Content-Encoding: gzip
@@ -454,7 +483,9 @@ void WebProcessor::SendBodyResponse(const char* body, int bodyLen, const char* c
 
 	BString<1024> responseHeader(RESPONSE_HEADER,
 		m_origin.Str(),
-		m_serverAuthToken[m_userAccess], bodyLen, *contentTypeHeader,
+		g_Options->GetFormAuth() ? "form" : "http",
+		m_authorized ? m_serverAuthToken[m_userAccess] : "",
+		bodyLen, *contentTypeHeader,
 		gzip ? "Content-Encoding: gzip\r\n" : "",
 		Util::VersionRevision());
 
