@@ -53,12 +53,15 @@ private:
 	CString m_url;
 	EHttpMethod m_httpMethod;
 	EUserAccess m_userAccess;
+	bool m_rpcRequest;
+	bool m_authorized;
 	bool m_gzip;
 	CString m_origin;
 	int m_contentLen;
 	char m_authInfo[256+1];
 	char m_authToken[48+1];
 	static char m_serverAuthToken[3][48+1];
+	CString m_forwardedFor;
 
 	void Dispatch();
 	void SendAuthResponse();
