@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -417,7 +417,7 @@ CString FileSystem::MakeValidFilename(const char* filename, bool allowSlashes)
 CString FileSystem::MakeUniqueFilename(const char* destDir, const char* basename)
 {
 	CString result;
-	result.Format("%s%c%s", destDir, (int)PATH_SEPARATOR, basename);
+	result.Format("%s%c%s", destDir, PATH_SEPARATOR, basename);
 
 	int dupeNumber = 0;
 	while (FileExists(result))
@@ -442,12 +442,12 @@ CString FileSystem::MakeUniqueFilename(const char* destDir, const char* basename
 				}
 			}
 
-			result.Format("%s%c%s.duplicate%d%s", destDir, (int)PATH_SEPARATOR,
+			result.Format("%s%c%s.duplicate%d%s", destDir, PATH_SEPARATOR,
 				*filenameWithoutExt, dupeNumber, extension);
 		}
 		else
 		{
-			result.Format("%s%c%s.duplicate%d", destDir, (int)PATH_SEPARATOR,
+			result.Format("%s%c%s.duplicate%d", destDir, PATH_SEPARATOR,
 				basename, dupeNumber);
 		}
 	}
@@ -527,13 +527,6 @@ bool FileSystem::FileExists(const char* filename)
 #endif
 }
 
-bool FileSystem::FileExists(const char* path, const char* filenameWithoutPath)
-{
-	BString<1024> fullFilename("%s%c%s", path, (int)PATH_SEPARATOR, filenameWithoutPath);
-	bool exists = FileExists(fullFilename);
-	return exists;
-}
-
 bool FileSystem::DirectoryExists(const char* dirFilename)
 {
 #ifdef WIN32
@@ -580,6 +573,37 @@ bool FileSystem::RemoveDirectory(const char* dirFilename)
 #endif
 }
 
+/* Delete directory which is empty or contains only hidden files or directories (whose names start with dot) */
+bool FileSystem::DeleteDirectory(const char* dirFilename)
+{
+	if (RemoveDirectory(dirFilename))
+	{
+		return true;
+	}
+
+	// check if directory contains only hidden files (whose names start with dot)
+	{
+		DirBrowser dir(dirFilename);
+		while (const char* filename = dir.Next())
+		{
+			if (*filename != '.')
+			{
+				// calling RemoveDirectory to set correct errno
+				return RemoveDirectory(dirFilename);
+			}
+		}
+	} // make sure "DirBrowser dir" is destroyed (and has closed its handle) before we trying to delete the directory
+
+	CString errmsg;
+	if (!DeleteDirectoryWithContent(dirFilename, errmsg))
+	{
+		// calling RemoveDirectory to set correct errno
+		return RemoveDirectory(dirFilename);
+	}
+
+	return true;
+}
+
 bool FileSystem::DeleteDirectoryWithContent(const char* dirFilename, CString& errmsg)
 {
 	errmsg.Clear();
@@ -999,7 +1023,7 @@ CString FileSystem::WidePathToUtfPath(const wchar_t* wpath)
 #ifdef WIN32
 DirBrowser::DirBrowser(const char* path)
 {
-	BString<1024> mask("%s%c*.*", path, (int)PATH_SEPARATOR);
+	BString<1024> mask("%s%c*.*", path, PATH_SEPARATOR);
 	m_file = FindFirstFileW(FileSystem::UtfPathToWidePath(mask), &m_findData);
 	m_first = true;
 }
@@ -1168,7 +1192,7 @@ int64 DiskFile::Position()
 	return ftell(m_file);
 }
 
-int64 DiskFile::Seek(int64 position, ESeekOrigin origin)
+bool DiskFile::Seek(int64 position, ESeekOrigin origin)
 {
 	return fseek(m_file, position,
 		origin == soCur ? SEEK_CUR :
