@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
- *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -198,14 +198,15 @@ void ScriptController::PrepareEnvOptions(const char* stripPrefix)
 
 	for (Options::OptEntry& optEntry : g_Options->GuardOptEntries())
 	{
+		const char* value = GetOptValue(optEntry.GetName(), optEntry.GetValue());
 		if (stripPrefix && !strncmp(optEntry.GetName(), stripPrefix, prefixLen) &&
 			(int)strlen(optEntry.GetName()) > prefixLen)
 		{
-			SetEnvVarSpecial("NZBPO", optEntry.GetName() + prefixLen, optEntry.GetValue());
+			SetEnvVarSpecial("NZBPO", optEntry.GetName() + prefixLen, value);
 		}
 		else if (!stripPrefix)
 		{
-			SetEnvVarSpecial("NZBOP", optEntry.GetName(), optEntry.GetValue());
+			SetEnvVarSpecial("NZBOP", optEntry.GetName(), value);
 		}
 	}
 }
@@ -287,6 +288,7 @@ void ScriptController::PrepareArgs()
 					if (FormatMessage(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ARGUMENT_ARRAY, command, 0, 0,
 						m_cmdLine, sizeof(m_cmdLine), (va_list*)args))
 					{
+						Util::TrimRight(Util::ReduceStr(m_cmdLine, "*", ""));
 						debug("CmdLine: %s", m_cmdLine);
 						return;
 					}
@@ -304,6 +306,7 @@ int ScriptController::Execute()
 	PrepareEnvOptions(nullptr);
 	PrepareArgs();
 
+	m_completed = false;
 	int exitCode = 0;
 
 #ifdef CHILD_WATCHDOG
@@ -312,9 +315,11 @@ int ScriptController::Execute()
 	{
 #endif
 
-	int pipein = StartProcess();
+	int pipein = -1, pipeout = -1;
+	StartProcess(&pipein, &pipeout);
 	if (pipein == -1)
 	{
+		m_completed = true;
 		return -1;
 	}
 
@@ -322,11 +327,28 @@ int ScriptController::Execute()
 	m_readpipe = fdopen(pipein, "r");
 	if (!m_readpipe)
 	{
-		PrintMessage(Message::mkError, "Could not open pipe to %s", m_infoName);
+		PrintMessage(Message::mkError, "Could not open read pipe to %s", *m_infoName);
 		close(pipein);
+		close(pipeout);
+		m_completed = true;
 		return -1;
 	}
 
+	m_writepipe = 0;
+	if (m_needWrite)
+	{
+		// open the write end
+		m_writepipe = fdopen(pipeout, "w");
+		if (!m_writepipe)
+		{
+			PrintMessage(Message::mkError, "Could not open write pipe to %s", *m_infoName);
+			close(pipein);
+			close(pipeout);
+			m_completed = true;
+			return -1;
+		}
+	}
+
 #ifdef CHILD_WATCHDOG
 	debug("Creating child watchdog");
 	ChildWatchDog watchDog;
@@ -381,14 +403,19 @@ int ScriptController::Execute()
 		fclose(m_readpipe);
 	}
 
+	if (m_writepipe)
+	{
+		fclose(m_writepipe);
+	}
+
 	if (m_terminated && m_infoName)
 	{
-		warn("Interrupted %s", m_infoName);
+		warn("Interrupted %s", *m_infoName);
 	}
 
 	exitCode = 0;
 
-	if (!m_terminated && !m_detached)
+	if (!m_detached)
 	{
 		exitCode = WaitProcess();
 #ifndef WIN32
@@ -404,7 +431,7 @@ int ScriptController::Execute()
 #endif
 
 	debug("Exit code %i", exitCode);
-
+	m_completed = true;
 	return exitCode;
 }
 
@@ -427,7 +454,7 @@ void ScriptController::BuildCommandLine(char* cmdLineBuf, int bufSize)
 /*
 * Returns file descriptor of the read-pipe or -1 on error.
 */
-int ScriptController::StartProcess()
+void ScriptController::StartProcess(int* pipein, int* pipeout)
 {
 	CString workingDir = m_workingDir;
 	if (workingDir.Empty())
@@ -453,28 +480,34 @@ int ScriptController::StartProcess()
 	}
 
 	// create pipes to write and read data
-	HANDLE readPipe, writePipe;
+	HANDLE readPipe, readProcPipe;
+	HANDLE writePipe = 0, writeProcPipe = 0;
 	SECURITY_ATTRIBUTES securityAttributes = { 0 };
 	securityAttributes.nLength = sizeof(securityAttributes);
 	securityAttributes.bInheritHandle = TRUE;
 
-	CreatePipe(&readPipe, &writePipe, &securityAttributes, 0);
-
+	CreatePipe(&readPipe, &readProcPipe, &securityAttributes, 0);
 	SetHandleInformation(readPipe, HANDLE_FLAG_INHERIT, 0);
 
+	if (m_needWrite)
+	{
+		CreatePipe(&writeProcPipe, &writePipe, &securityAttributes, 0);
+		SetHandleInformation(writePipe, HANDLE_FLAG_INHERIT, 0);
+	}
+
 	STARTUPINFOW startupInfo = { 0 };
 	startupInfo.cb = sizeof(startupInfo);
 	startupInfo.dwFlags = STARTF_USESTDHANDLES;
-	startupInfo.hStdInput = 0;
-	startupInfo.hStdOutput = writePipe;
-	startupInfo.hStdError = writePipe;
+	startupInfo.hStdInput = writeProcPipe;
+	startupInfo.hStdOutput = readProcPipe;
+	startupInfo.hStdError = readProcPipe;
 
 	PROCESS_INFORMATION processInfo = { 0 };
 
 	std::unique_ptr<wchar_t[]> environmentStrings = m_environmentStrings.GetStrings();
 
 	BOOL ok = CreateProcessW(nullptr, WString(cmdLine), nullptr, nullptr, TRUE,
-		NORMAL_PRIORITY_CLASS | CREATE_NO_WINDOW | CREATE_UNICODE_ENVIRONMENT,
+		NORMAL_PRIORITY_CLASS | CREATE_NEW_PROCESS_GROUP | CREATE_UNICODE_ENVIRONMENT,
 		environmentStrings.get(), wideWorkingDir, &startupInfo, &processInfo);
 	if (!ok)
 	{
@@ -483,11 +516,11 @@ int ScriptController::StartProcess()
 		errMsg[255 - 1] = '\0';
 		if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nullptr, errCode, 0, errMsg, 255, nullptr))
 		{
-			PrintMessage(Message::mkError, "Could not start %s: %s", m_infoName, errMsg);
+			PrintMessage(Message::mkError, "Could not start %s: %s", *m_infoName, errMsg);
 		}
 		else
 		{
-			PrintMessage(Message::mkError, "Could not start %s: error %i", m_infoName, errCode);
+			PrintMessage(Message::mkError, "Could not start %s: error %i", *m_infoName, errCode);
 		}
 		if (!FileSystem::FileExists(script))
 		{
@@ -498,32 +531,48 @@ int ScriptController::StartProcess()
 			PrintMessage(Message::mkError, "Could not build short path for %s", workingDir);
 		}
 		CloseHandle(readPipe);
+		CloseHandle(readProcPipe);
 		CloseHandle(writePipe);
-		return -1;
+		CloseHandle(writeProcPipe);
+		return;
 	}
 
 	debug("Child Process-ID: %i", (int)processInfo.dwProcessId);
 
 	m_processId = processInfo.hProcess;
+	m_dwProcessId = processInfo.dwProcessId;
 
-	// close unused "write" end
-	CloseHandle(writePipe);
+	// close unused pipe ends
+	CloseHandle(readProcPipe);
+	CloseHandle(writeProcPipe);
 
-	int pipein = _open_osfhandle((intptr_t)readPipe, _O_RDONLY);
-	return pipein;
+	*pipein = _open_osfhandle((intptr_t)readPipe, _O_RDONLY);
+	if (m_needWrite)
+	{
+		*pipeout = _open_osfhandle((intptr_t)writePipe, _O_WRONLY);
+	}
 
 #else
 
-	int p[2];
-	int pipein;
-	int pipeout;
+	int pin[] = {0, 0};
+	int pout[] = {0, 0};
 
-	// create the pipe
-	if (pipe(p))
+	// create the pipes
+	if (pipe(pin))
 	{
-		PrintMessage(Message::mkError, "Could not open pipe: errno %i", errno);
-		return -1;
+		PrintMessage(Message::mkError, "Could not open read pipe: errno %i", errno);
+		return;
 	}
+	if (m_needWrite && pipe(pout))
+	{
+		PrintMessage(Message::mkError, "Could not open write pipe: errno %i", errno);
+		close(pin[0]);
+		close(pin[1]);
+		return;
+	}
+
+	*pipein = pin[0];
+	*pipeout = pout[1];
 
 	std::vector<char*> environmentStrings = m_environmentStrings.GetStrings();
 	char** envdata = environmentStrings.data();
@@ -533,18 +582,20 @@ int ScriptController::StartProcess()
 	args.emplace_back(nullptr);
 	char* const* argdata = (char* const*)args.data();
 
-	pipein = p[0];
-	pipeout = p[1];
-
 	debug("forking");
 	pid_t pid = fork();
 
 	if (pid == -1)
 	{
-		PrintMessage(Message::mkError, "Could not start %s: errno %i", m_infoName, errno);
-		close(pipein);
-		close(pipeout);
-		return -1;
+		PrintMessage(Message::mkError, "Could not start %s: errno %i", *m_infoName, errno);
+		close(pin[0]);
+		close(pin[1]);
+		if (m_needWrite)
+		{
+			close(pout[0]);
+			close(pout[1]);
+		}
+		return;
 	}
 	else if (pid == 0)
 	{
@@ -557,14 +608,19 @@ int ScriptController::StartProcess()
 		// create new process group (see Terminate() where it is used)
 		setsid();
 
-		// close up the "read" end
-		close(pipein);
-
 		// make the pipeout to be the same as stdout and stderr
-		dup2(pipeout, 1);
-		dup2(pipeout, 2);
+		dup2(pin[1], 1);
+		dup2(pin[1], 2);
+		close(pin[0]);
+		close(pin[1]);
 
-		close(pipeout);
+		if (m_needWrite)
+		{
+			// make the pipein to be the same as stdin
+			dup2(pout[0], 0);
+			close(pout[0]);
+			close(pout[1]);
+		}
 
 #ifdef CHILD_WATCHDOG
 		write(1, "\n", 1);
@@ -604,17 +660,20 @@ int ScriptController::StartProcess()
 
 	m_processId = pid;
 
-	// close unused "write" end
-	close(pipeout);
+	// close unused pipe ends
+	close(pin[1]);
+	if (m_needWrite)
+	{
+		close(pout[0]);
+	}
 #endif
-
-	return pipein;
 }
 
 int ScriptController::WaitProcess()
 {
 #ifdef WIN32
-	WaitForSingleObject(m_processId, INFINITE);
+	// wait max 60 seconds for terminated processes
+	WaitForSingleObject(m_processId, m_terminated ? 60 * 1000 : INFINITE);
 	DWORD exitCode = 0;
 	GetExitCodeProcess(m_processId, &exitCode);
 	return exitCode;
@@ -632,22 +691,11 @@ int ScriptController::WaitProcess()
 
 void ScriptController::Terminate()
 {
-	debug("Stopping %s", m_infoName);
+	debug("Stopping %s", *m_infoName);
 	m_terminated = true;
 
 #ifdef WIN32
-	BOOL ok = TerminateProcess(m_processId, -1);
-	if (ok)
-	{
-		// wait 60 seconds for process to terminate
-		WaitForSingleObject(m_processId, 60 * 1000);
-	}
-	else
-	{
-		DWORD exitCode = 0;
-		GetExitCodeProcess(m_processId, &exitCode);
-		ok = exitCode != STILL_ACTIVE;
-	}
+	BOOL ok = TerminateProcess(m_processId, -1) || m_completed;
 #else
 	pid_t killId = m_processId;
 	if (getpgid(killId) == killId)
@@ -655,19 +703,19 @@ void ScriptController::Terminate()
 		// if the child process has its own group (setsid() was successful), kill the whole group
 		killId = -killId;
 	}
-	bool ok = killId && kill(killId, SIGKILL) == 0;
+	bool ok = (killId && kill(killId, SIGKILL) == 0) || m_completed;
 #endif
 
 	if (ok)
 	{
-		debug("Terminated %s", m_infoName);
+		debug("Terminated %s", *m_infoName);
 	}
 	else
 	{
-		error("Could not terminate %s", m_infoName);
+		error("Could not terminate %s", *m_infoName);
 	}
 
-	debug("Stopped %s", m_infoName);
+	debug("Stopped %s", *m_infoName);
 }
 
 void ScriptController::TerminateAll()
@@ -677,14 +725,45 @@ void ScriptController::TerminateAll()
 	{
 		if (script->m_processId && !script->m_detached)
 		{
+			// send break signal and wait up to 5 seconds for graceful termination
+			if (script->Break())
+			{
+				time_t curtime = Util::CurrentTime();
+				while (!script->m_completed && std::abs(curtime - Util::CurrentTime()) <= 10)
+				{
+					usleep(100 * 1000);
+				}
+			}
 			script->Terminate();
 		}
 	}
 }
 
+bool ScriptController::Break()
+{
+	debug("Sending break signal to %s", *m_infoName);
+
+#ifdef WIN32
+	BOOL ok = GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, m_dwProcessId);
+#else
+	bool ok = kill(m_processId, SIGINT) == 0;
+#endif
+
+	if (ok)
+	{
+		debug("Sent break signal to %s", *m_infoName);
+	}
+	else
+	{
+		warn("Could not send break signal to %s", *m_infoName);
+	}
+
+	return ok;
+}
+
 void ScriptController::Detach()
 {
-	debug("Detaching %s", m_infoName);
+	debug("Detaching %s", *m_infoName);
 	m_detached = true;
 	FILE* readpipe = m_readpipe;
 	m_readpipe = nullptr;
@@ -787,3 +866,9 @@ void ScriptController::PrintMessage(Message::EKind kind, const char* format, ...
 		AddMessage(kind, tmp2);
 	}
 }
+
+void ScriptController::Write(const char* str)
+{
+	fwrite(str, 1, strlen(str), m_writepipe);
+	fflush(m_writepipe);
+}
