@@ -1,8 +1,8 @@
 #!/bin/sh
 #
-#  This file is part of nzbget
+#  This file is part of nzbget. See <http://nzbget.net>.
 #
-#  Copyright (C) 2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
+#  Copyright (C) 2015-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -15,8 +15,7 @@
 #  GNU General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software
-#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
 if test "$NZBUP_PROCESSID" = ""; then
@@ -136,7 +135,7 @@ if test "$?" != "0"; then
 fi
 while true
 do
-    RUNNING=`ps $PSOPT | sed -n "s/^\s*$NZBUP_PROCESSID\s.*/&/p"` 
+    RUNNING=`ps $PSOPT | sed -n "s/^[\s ]*$NZBUP_PROCESSID[\s ].*/&/p"` 
     if test "$RUNNING" = ""; then
         break
     fi
