@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget. See <http://nzbget.net>.
  *
- *  Copyright (C) 2013-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2013-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -120,18 +120,11 @@ var FeedDialog = (new function($)
 			{
 				filterInput: '#FeedDialog_ItemTable_filter',
 				pagerContainer: '#FeedDialog_ItemTable_pager',
-				headerCheck: '#FeedDialog_ItemTable > thead > tr:first-child',
+				rowSelect: UISettings.rowSelect,
 				pageSize: pageSize,
-				hasHeader: true,
 				renderCellCallback: itemsTableRenderCellCallback
 			});
 
-		$ItemTable.on('click', UISettings.rowSelect ? 'tbody tr' : 'tbody div.check',
-			function(event) { $ItemTable.fasttable('itemCheckClick', UISettings.rowSelect ? this : this.parentNode.parentNode, event); });
-		$ItemTable.on('click', 'thead div.check',
-			function() { $ItemTable.fasttable('titleCheckClick') });
-		$ItemTable.on('mousedown', Util.disableShiftMouseDown);
-
 		$FeedDialog.on('hidden', function()
 		{
 			// cleanup
@@ -155,14 +148,15 @@ var FeedDialog = (new function($)
 	{
 		Refresher.pause();
 
-		$ItemTable.fasttable('update', []);
-
 		enableAllButtons();
 		$FeedDialog.restoreTab();
 
 		$('#FeedDialog_ItemTable_filter').val('');
 		$('#FeedDialog_ItemTable_pagerBlock').hide();
 
+		$ItemTable.fasttable('update', []);
+		$ItemTable.fasttable('applyFilter', '');
+
 		items = null;
 
 		curFilter = 'ALL';
@@ -195,6 +189,14 @@ var FeedDialog = (new function($)
 			RPC.call('previewfeed', [id, name, url, filter, feedBacklog, feedPauseNzb, feedCategory,
 				feedPriority, feedInterval, feedScript, false, 0, ''], itemsLoaded, feedFailure);
 		}
+
+		if (!UISettings.miniTheme)
+		{
+			$('#FeedDialog_TableBlock').removeClass('modal-inner-scroll');
+			$('#FeedDialog_TableBlock').css('top', '');
+			$('#FeedDialog_TableBlock').css('top', $('#FeedDialog_TableBlock').position().top);
+			$('#FeedDialog_TableBlock').addClass('modal-inner-scroll');
+		}
 	}
 
 	function feedFailure(res)
@@ -272,6 +274,12 @@ var FeedDialog = (new function($)
 			else
 			{
 				var info = '<div class="check img-check"></div><span class="row-title">' + name + '</span>' + ' ' + status;
+				if (item.Category !== '')
+				{
+					info += ' <span class="label label-info">' + item.Category + '</span>';
+				}
+				info += ' <span class="label label-info">' + age + '</span>' +
+					' <span class="label label-info">' + size + '</span>';
 				fields = [info];
 			}
 
@@ -465,7 +473,6 @@ var FeedFilterDialog = (new function($)
 				pagerContainer: '#FeedFilterDialog_ItemTable_pager',
 				headerCheck: '',
 				pageSize: pageSize,
-				hasHeader: true,
 				renderCellCallback: itemsTableRenderCellCallback
 			});
 
@@ -615,7 +622,7 @@ var FeedFilterDialog = (new function($)
 		var differentFilenames = false;
 
 		var filter = $FilterInput.val().split('\n');
-		
+
 		var data = [];
 
 		for (var i=0; i < items.length; i++)
@@ -796,7 +803,7 @@ var FeedFilterDialog = (new function($)
 		$RematchIcon.toggleClass('icon-process', !autoUpdate);
 		$RematchIcon.toggleClass('icon-process-auto', autoUpdate);
 	}
-	
+
 	function filterKeyPress(event)
 	{
 		if (event.which == 37)
@@ -805,7 +812,7 @@ var FeedFilterDialog = (new function($)
 			alert('Percent character (%) cannot be part of a filter because it is used\nas line separator when saving filter into configuration file.');
 		}
 	}
-	
+
 	/*** SPLITTER ***/
 
 	function splitterMouseDown(e)
@@ -861,7 +868,7 @@ var FeedFilterDialog = (new function($)
 			windowResized();
 		}
 	}
-	
+
 	function windowResized()
 	{
 		if (!UISettings.miniTheme)
@@ -875,7 +882,7 @@ var FeedFilterDialog = (new function($)
 			splitterMove(newWidth - oldWidth);
 		}
 	}
-	
+
 	/*** LINE SELECTION ***/
 
 	this.selectRule = function(rule)
@@ -913,7 +920,7 @@ var FeedFilterDialog = (new function($)
 
 	// Idea and portions of code from LinedTextArea plugin by Alan Williamson
 	// http://files.aw20.net/jquery-linedtextarea/jquery-linedtextarea.html
-	
+
 	function initLines()
 	{
 		showLines = !UISettings.miniTheme;
@@ -923,7 +930,7 @@ var FeedFilterDialog = (new function($)
 			$FilterInput.scroll(updateLines);
 		}
 	}
-	
+
 	function updateLines()
 	{
 		if (!UISettings.miniTheme && showLines)
