@@ -1,7 +1,7 @@
 /*
  * This file is part of nzbget. See <http://nzbget.net>.
  *
- * Copyright (C) 2012-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ * Copyright (C) 2012-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
 /*
  * In this module:
  *   1) History tab;
- *   2) Functions for html generation for history, also used from other modules (edit dialog).
+ *   2) Functions for html generation for history, also used from other modules (edit dialog);
+ *   3) Popup menus in history list.
  */
 
 /*** HISTORY TAB AND EDIT HISTORY DIALOG **********************************************/
@@ -34,6 +35,7 @@ var History = (new function($)
 	var $HistoryTabBadge;
 	var $HistoryTabBadgeEmpty;
 	var $HistoryRecordsPerPage;
+	var $CategoryMenu;
 
 	// State
 	var history;
@@ -51,6 +53,7 @@ var History = (new function($)
 		$HistoryTabBadge = $('#HistoryTabBadge');
 		$HistoryTabBadgeEmpty = $('#HistoryTabBadgeEmpty');
 		$HistoryRecordsPerPage = $('#HistoryRecordsPerPage');
+		$CategoryMenu = $('#HistoryCategoryMenu');
 
 		var recordsPerPage = UISettings.read('HistoryRecordsPerPage', 10);
 		$HistoryRecordsPerPage.val(recordsPerPage);
@@ -58,14 +61,15 @@ var History = (new function($)
 
 		$HistoryTable.fasttable(
 			{
-				filterInput: $('#HistoryTable_filter'),
-				filterClearButton: $("#HistoryTable_clearfilter"),
-				pagerContainer: $('#HistoryTable_pager'),
-				infoContainer: $('#HistoryTable_info'),
-				headerCheck: $('#HistoryTable > thead > tr:first-child'),
+				filterInput: '#HistoryTable_filter',
+				filterClearButton: '#HistoryTable_clearfilter',
+				pagerContainer: '#HistoryTable_pager',
+				infoContainer: '#HistoryTable_info',
+				rowSelect: UISettings.rowSelect,
 				pageSize: recordsPerPage,
 				maxPages: UISettings.miniTheme ? 1 : 5,
 				pageDots: !UISettings.miniTheme,
+				shortcuts: true,
 				fillFieldsCallback: fillFieldsCallback,
 				filterCallback: filterCallback,
 				renderCellCallback: renderCellCallback,
@@ -73,11 +77,11 @@ var History = (new function($)
 			});
 
 		$HistoryTable.on('click', 'a', editClick);
-		$HistoryTable.on('click', UISettings.rowSelect ? 'tbody tr' : 'tbody div.check',
-			function(event) { $HistoryTable.fasttable('itemCheckClick', UISettings.rowSelect ? this : this.parentNode.parentNode, event); });
-		$HistoryTable.on('click', 'thead div.check',
-			function() { $HistoryTable.fasttable('titleCheckClick') });
-		$HistoryTable.on('mousedown', Util.disableShiftMouseDown);
+		$HistoryTable.on('click', 'td:nth-child(2).dropdown-cell > div', statusClick);
+		$HistoryTable.on('click', 'td:nth-child(5).dropdown-cell > div:not(.dropdown-disabled)', categoryClick);
+		$CategoryMenu.on('click', 'a', categoryMenuClick);
+
+		HistoryActionsMenu.init();
 	}
 
 	this.applyTheme = function()
@@ -188,14 +192,14 @@ var History = (new function($)
 
 		var status = HistoryUI.buildStatus(hist);
 
-		var name = '<a href="#" histid="' + hist.ID + '">' + Util.textToHtml(Util.formatNZBName(hist.Name)) + '</a>';
-		var dupe = DownloadsUI.buildDupe(hist.DupeKey, hist.DupeScore, hist.DupeMode);
-		var category = '';
+		var name = '<a href="#" data-nzbid="' + hist.ID + '">' + Util.textToHtml(Util.formatNZBName(hist.Name)) + '</a>';
+		name += DownloadsUI.buildEncryptedLabel(hist.Kind === 'NZB' ? hist.Parameters : []);
 
-		if (hist.Kind !== 'DUP')
-		{
-			var category = Util.textToHtml(hist.Category);
-		}
+		var dupe = DownloadsUI.buildDupe(hist.DupeKey, hist.DupeScore, hist.DupeMode);
+		var category = hist.Kind !== 'DUP' ?
+			(hist.Category !== '' ? Util.textToHtml(hist.Category) : '<span class="none-category">None</span>')
+			: '';
+		var backup = hist.Kind === 'NZB' ? DownloadsUI.buildBackupLabel(hist) : '';
 
 		if (hist.Kind === 'URL')
 		{
@@ -208,12 +212,14 @@ var History = (new function($)
 
 		if (!UISettings.miniTheme)
 		{
-			item.fields = ['<div class="check img-check"></div>', status, item.data.time, name + dupe, category, item.data.age, item.data.size];
+			status = '<div data-nzbid="' + hist.NZBID + '">' + status + '</div>';
+			category = '<div data-nzbid="' + hist.NZBID + '"' +  (hist.Kind === 'DUP' ? ' class="dropdown-disabled"' : '') + '>' + category + '</div>';
+			item.fields = ['<div class="check img-check"></div>', status, item.data.time, name + dupe + backup, category, item.data.age, item.data.size];
 		}
 		else
 		{
 			var info = '<div class="check img-check"></div><span class="row-title">' + name + '</span>' + dupe +
-				' ' + status + ' <span class="label">' + item.data.time + '</span>';
+				' ' + status + backup + ' <span class="label">' + item.data.time + '</span>';
 			if (category)
 			{
 				info += ' <span class="label label-status">' + category + '</span>';
@@ -228,11 +234,19 @@ var History = (new function($)
 
 	function renderCellCallback(cell, index, item)
 	{
-		if (index === 2)
+		if (index === 1 || index === 4)
 		{
-			cell.className = 'text-center';
+			cell.className = !UISettings.miniTheme ? 'dropdown-cell' : '';
 		}
-		else if (index === 5 || index === 6)
+		else if (index === 2)
+		{
+			cell.className = 'text-center' + (!UISettings.miniTheme ? ' dropafter-cell' : '');
+		}
+		else if (index === 5)
+		{
+			cell.className = 'text-right' + (!UISettings.miniTheme ? ' dropafter-cell' : '');
+		}
+		else if (index === 6)
 		{
 			cell.className = 'text-right';
 		}
@@ -264,6 +278,9 @@ var History = (new function($)
 			return;
 		}
 
+		var pageCheckedCount = $HistoryTable.fasttable('pageCheckedCount');
+		var checkedPercentage = Util.round0(checkedCount / history.length * 100);
+		
 		var hasNzb = false;
 		var hasUrl = false;
 		var hasDup = false;
@@ -285,7 +302,7 @@ var History = (new function($)
 		{
 			case 'DELETE':
 				notification = '#Notif_History_Deleted';
-				HistoryUI.deleteConfirm(historyAction, hasNzb, hasDup, hasFailed, true, checkedCount);
+				HistoryUI.deleteConfirm(historyAction, hasNzb, hasDup, hasFailed, true, checkedCount, pageCheckedCount, checkedPercentage);
 				break;
 
 			case 'REPROCESS':
@@ -321,7 +338,7 @@ var History = (new function($)
 				}
 				notification = '#Notif_History_Marked';
 
-				ConfirmDialog.showModal(action === 'MARKSUCCESS' ? 'HistoryEditSuccessConfirmDialog' : 
+				ConfirmDialog.showModal(action === 'MARKSUCCESS' ? 'HistoryEditSuccessConfirmDialog' :
 					action === 'MARKGOOD' ? 'HistoryEditGoodConfirmDialog' : 'HistoryEditBadConfirmDialog',
 					function () // action
 					{
@@ -342,14 +359,9 @@ var History = (new function($)
 	{
 		Refresher.pause();
 
-		var ids = [];
-		var checkedRows = $HistoryTable.fasttable('checkedRows');
-		for (var id in checkedRows)
-		{
-			ids.push(parseInt(id));
-		}		
-		
-		RPC.call('editqueue', [command, 0, '', ids], function()
+		var ids = buildContextIdList();
+
+		RPC.call('editqueue', [command, '', ids], function()
 		{
 			editCompleted();
 		});
@@ -370,27 +382,86 @@ var History = (new function($)
 		e.preventDefault();
 		e.stopPropagation();
 
-		var histid = $(this).attr('histid');
+		var histid = $(this).attr('data-nzbid');
+		var area = $(this).attr('data-area');
 		$(this).blur();
 
-		var hist = null;
-
-		// find history object
-		for (var i=0; i<history.length; i++)
-		{
-			var gr = history[i];
-			if (gr.ID == histid)
-			{
-				hist = gr;
-				break;
-			}
-		}
+		var hist = findHist(histid);
 		if (hist == null)
 		{
 			return;
 		}
 
-		HistoryEditDialog.showModal(hist);
+		HistoryEditDialog.showModal(hist, area);
+	}
+
+	function findHist(nzbid)
+	{
+		for (var i=0; i<history.length; i++)
+		{
+			var gr = history[i];
+			if (gr.NZBID == nzbid)
+			{
+				return gr;
+			}
+		}
+		return null;
+	}
+
+	function buildContextIdList(hist)
+	{
+		var editIds = [];
+		var checkedRows = $HistoryTable.fasttable('checkedRows');
+		for (var id in checkedRows)
+		{
+			editIds.push(parseInt(id));
+		}
+		if (hist !== undefined && editIds.indexOf(hist.NZBID) === -1)
+		{
+			editIds = [hist.NZBID];
+		}
+		return editIds;
+	}
+	this.buildContextIdList = buildContextIdList;
+
+	function statusClick(e)
+	{
+		e.preventDefault();
+		e.stopPropagation();
+		var hist = findHist($(this).attr('data-nzbid'));
+
+		HistoryActionsMenu.showPopupMenu(hist, 'left',
+			{ left: $(this).offset().left - 30, top: $(this).offset().top,
+				width: $(this).width() + 30, height: $(this).outerHeight() - 2 },
+			function(_notification) { notification = _notification; },
+			editCompleted);
+	}
+
+	function categoryClick(e)
+	{
+		e.preventDefault();
+		e.stopPropagation();
+		DownloadsUI.fillCategoryMenu($CategoryMenu);
+
+		var hist = findHist($(this).attr('data-nzbid'));
+		var editIds = buildContextIdList(hist);
+		$CategoryMenu.data('nzbids', editIds);
+		DownloadsUI.updateContextWarning($CategoryMenu, editIds);
+		$('i', $CategoryMenu).removeClass('icon-ok').addClass('icon-empty');
+		$('li[data="' + hist.Category + '"] i', $CategoryMenu).addClass('icon-ok');
+
+		Frontend.showPopupMenu($CategoryMenu, 'bottom-left',
+			{ left: $(this).offset().left - 30, top: $(this).offset().top,
+				width: $(this).width() + 30, height: $(this).outerHeight() - 2 });
+	}
+
+	function categoryMenuClick(e)
+	{
+		e.preventDefault();
+		var category = $(this).parent().attr('data');
+		var nzbids = $CategoryMenu.data('nzbids');
+		notification = '#Notif_History_Changed';
+		RPC.call('editqueue', ['HistorySetCategory', category, nzbids], editCompleted);
 	}
 
 	function filterCallback(item)
@@ -449,6 +520,26 @@ var History = (new function($)
 		Refresher.update();
 	}
 
+	this.processShortcut = function(key)
+	{
+		switch (key)
+		{
+			case 'D': case 'Delete': case 'Meta+Backspace': History.actionClick('DELETE'); return true;
+			case 'P': History.actionClick('REPROCESS'); return true;
+			case 'N': History.actionClick('REDOWNLOAD'); return true;
+			case 'M': History.actionClick('MARKSUCCESS'); return true;
+			case 'G': History.actionClick('MARKGOOD'); return true;
+			case 'B': History.actionClick('MARKBAD'); return true;
+			case 'A': History.filter('ALL'); return true;
+			case 'S': History.filter('SUCCESS'); return true;
+			case 'F': History.filter('FAILURE'); return true;
+			case 'L': History.filter('DELETED'); return true;
+			case 'U': History.filter('DUPE'); return true;
+			case 'H': History.dupClick(); return true;
+		}
+		return $HistoryTable.fasttable('processShortcut', key);
+	}
+
 }(jQuery));
 
 
@@ -494,8 +585,8 @@ var HistoryUI = (new function($)
 		}
 		return '<span class="label label-status ' + badgeClass + '">' + statusText + '</span>';
 	}
-	
-	this.deleteConfirm = function(actionCallback, hasNzb, hasDup, hasFailed, multi, selCount)
+
+	this.deleteConfirm = function(actionCallback, hasNzb, hasDup, hasFailed, multi, selCount, pageSelCount, selPercentage)
 	{
 		var dupeCheck = Options.option('DupeCheck') === 'yes';
 		var dialog = null;
@@ -517,12 +608,19 @@ var HistoryUI = (new function($)
 		{
 			var hide = $('#HistoryDeleteConfirmDialog_Hide', dialog).is(':checked');
 			var command = hasNzb && hide ? 'HistoryDelete' : 'HistoryFinalDelete';
-			actionCallback(command);
+			if (selCount - pageSelCount > 0 && selCount >= 50)
+			{
+				PurgeHistoryDialog.showModal(function(){actionCallback(command);}, selCount, selPercentage);
+			}
+			else
+			{
+				actionCallback(command);
+			}
 		}
 
 		ConfirmDialog.showModal('HistoryDeleteConfirmDialog', action, init, selCount);
 	}
-	
+
 	this.confirmMulti = function(multi)
 	{
 		if (multi === undefined || !multi)
@@ -534,3 +632,187 @@ var HistoryUI = (new function($)
 		}
 	}
 }(jQuery));
+
+/*** HISTORY ACTION MENU *************************************************************************/
+
+var HistoryActionsMenu = (new function()
+{
+	'use strict'
+
+	var $ActionsMenu;
+	var curHist;
+	var beforeCallback;
+	var completedCallback;
+	var editIds;
+
+	this.init = function()
+	{
+		$ActionsMenu = $('#HistoryActionsMenu');
+		$('#HistoryActions_Delete').click(itemDelete);
+		$('#HistoryActions_Return, #HistoryActions_ReturnURL').click(itemReturn);
+		$('#HistoryActions_Reprocess').click(itemReprocess);
+		$('#HistoryActions_Redownload').click(itemRedownload);
+		$('#HistoryActions_RetryFailed').click(itemRetryFailed);
+		$('#HistoryActions_MarkSuccess').click(itemSuccess);
+		$('#HistoryActions_MarkGood').click(itemGood);
+		$('#HistoryActions_MarkBad').click(itemBad);
+	}
+
+	this.showPopupMenu = function(hist, anchor, rect, before, completed)
+	{
+		curHist = hist;
+		beforeCallback = before;
+		completedCallback = completed;
+		editIds = History.buildContextIdList(hist);
+
+		// setup menu items
+		Util.show('#HistoryActions_Return', hist.RemainingFileCount > 0);
+		Util.show('#HistoryActions_ReturnURL', hist.Kind === 'URL');
+		Util.show('#HistoryActions_Redownload, #HistoryActions_Reprocess', hist.Kind === 'NZB');
+		Util.show('#HistoryActions_RetryFailed', hist.Kind === 'NZB' && hist.FailedArticles > 0 && hist.RetryData);
+		var dupeCheck = Options.option('DupeCheck') === 'yes';
+		Util.show('#HistoryActions_MarkSuccess', dupeCheck && ((hist.Kind === 'NZB' && hist.MarkStatus !== 'SUCCESS') || (hist.Kind === 'DUP' && hist.DupStatus !== 'SUCCESS')) &&
+			hist.Status.substr(0, 7) !== 'SUCCESS');
+		Util.show('#HistoryActions_MarkGood', dupeCheck && ((hist.Kind === 'NZB' && hist.MarkStatus !== 'GOOD') || (hist.Kind === 'DUP' && hist.DupStatus !== 'GOOD')));
+		Util.show('#HistoryActions_MarkBad', dupeCheck && hist.Kind !== 'URL');
+		DownloadsUI.updateContextWarning($ActionsMenu, editIds);
+
+		DownloadsUI.buildDNZBLinks(hist.Parameters ? hist.Parameters : [], 'HistoryActions_DNZB');
+		
+		Frontend.showPopupMenu($ActionsMenu, anchor, rect);
+	}
+	
+	function itemDelete(e)
+	{
+		e.preventDefault();
+		HistoryUI.deleteConfirm(doItemDelete, curHist.Kind === 'NZB', curHist.Kind === 'DUP',
+			curHist.ParStatus === 'FAILURE' || curHist.UnpackStatus === 'FAILURE' ||
+			curHist.DeleteStatus != 'NONE', false);
+	}
+
+	function execAction(command, notification)
+	{
+		function performAction()
+		{
+			RPC.call('editqueue', [command, '', editIds], completedCallback);
+		}
+
+		var async = beforeCallback(notification, performAction);
+		if (!async)
+		{
+			performAction();
+		}
+	}
+
+	function doItemDelete(command)
+	{
+		execAction(command, '#Notif_History_Deleted');
+	}
+
+	function itemReturn(e)
+	{
+		e.preventDefault();
+		execAction('HistoryReturn', '#Notif_History_Returned');
+	}
+
+	function itemRedownload(e)
+	{
+		e.preventDefault();
+		if (curHist.SuccessArticles > 0)
+		{
+			ConfirmDialog.showModal('HistoryEditRedownloadConfirmDialog', doItemRedownload,
+				function () { HistoryUI.confirmMulti(false); });
+		}
+		else
+		{
+			doItemRedownload();
+		}
+	}
+
+	function doItemRedownload()
+	{
+		execAction('HistoryRedownload', '#Notif_History_Returned');
+	}
+
+	function itemReprocess(e)
+	{
+		e.preventDefault();
+		execAction('HistoryProcess', '#Notif_History_Reprocess');
+	}
+
+	function itemRetryFailed(e)
+	{
+		e.preventDefault();
+		execAction('HistoryRetryFailed', '#Notif_History_RetryFailed');
+	}
+
+	function itemSuccess(e)
+	{
+		e.preventDefault();
+		ConfirmDialog.showModal('HistoryEditSuccessConfirmDialog', doItemSuccess,
+			function () { HistoryUI.confirmMulti(editIds.length > 1); });
+	}
+
+	function doItemSuccess()
+	{
+		execAction('HistoryMarkSuccess', '#Notif_History_Marked');
+	}
+
+	function itemGood(e)
+	{
+		e.preventDefault();
+		ConfirmDialog.showModal('HistoryEditGoodConfirmDialog', doItemGood,
+			function () { HistoryUI.confirmMulti(editIds.length > 1); });
+	}
+
+	function doItemGood()
+	{
+		execAction('HistoryMarkGood', '#Notif_History_Marked');
+	}
+
+	function itemBad(e)
+	{
+		e.preventDefault();
+		ConfirmDialog.showModal('HistoryEditBadConfirmDialog', doItemBad,
+			function () { HistoryUI.confirmMulti(editIds.length > 1); });
+	}
+
+	function doItemBad()
+	{
+		execAction('HistoryMarkBad', '#Notif_History_Marked');
+	}
+}(jQuery));
+
+/*** PURGE HISTORY DIALOG *****************************************************/
+
+var PurgeHistoryDialog = (new function($)
+{
+	'use strict';
+
+	// Controls
+	var $PurgeHistoryDialog;
+
+	// State
+	var actionCallback;
+
+	this.init = function()
+	{
+		$PurgeHistoryDialog = $('#PurgeHistoryDialog');
+	}
+
+	this.showModal = function(_actionCallback, count, percentage)
+	{
+		actionCallback = _actionCallback;
+		$('#PurgeHistoryDialog_count,#PurgeHistoryDialog_count2').text(count);
+		$('#PurgeHistoryDialog_percentage').text(percentage);
+		Util.centerDialog($PurgeHistoryDialog, true);
+		$PurgeHistoryDialog.modal({backdrop: 'static'});
+	}
+
+	this.delete = function(event)
+	{
+		event.preventDefault(); // avoid scrolling
+		$PurgeHistoryDialog.modal('hide');
+		actionCallback();
+	}
+}(jQuery));
