@@ -1,7 +1,7 @@
 /*
  * This file is part of nzbget. See <http://nzbget.net>.
  *
- * Copyright (C) 2012-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ * Copyright (C) 2012-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,9 +36,9 @@ var UISettings = (new function($)
 	/*** Web-interface configuration *************/
 
 	// Options having descriptions can be edited directly in web-interface on settings page.
-	
+
 	this.description = [];
-	
+
 	this.description['activityAnimation'] = 'Animation on play/pause button (yes, no).';
 	this.activityAnimation = true;
 
@@ -57,7 +57,7 @@ var UISettings = (new function($)
 
 	this.description['dupeBadges'] = 'Show badges with duplicate info in downloads and history (yes, no).';
 	this.dupeBadges = false;
-	
+
 	this.description['rowSelect'] = 'Select records by clicking on any part of the row, not just on the check mark (yes, no).';
 	this.rowSelect = false;
 
@@ -75,7 +75,7 @@ var UISettings = (new function($)
 		'%(VARNAME-)% - show variable value with hyphen inside parenthesis;\n' +
 		'%[VARNAME-]% - show variable value with hyphen inside square brackets.\n\n' +
 		'Examples:\n' +
-		' "%(COUNT-)% NZBGet" - show number of downloads in parenthesis followed by a hyphen; don\'t show "(0) - " if queue is empty;\n' + 
+		' "%(COUNT-)% NZBGet" - show number of downloads in parenthesis followed by a hyphen; don\'t show "(0) - " if queue is empty;\n' +
 		' "%PAUSE% %(COUNT-)% NZBGet" - as above but also show pause-indicator if paused;\n' +
 		' "%[COUNT]% %SPEED-% NZBGet" - show number of downloads and speed if not null (default setting).';
 	this.windowTitle = '%[COUNT]% %SPEED-% NZBGet';
@@ -83,7 +83,7 @@ var UISettings = (new function($)
 	this.description['refreshRetries'] = 'Number of refresh attempts if a communication error occurs (0-99).\n\n' +
 	  'If all attempts fail, an error is displayed and the automatic refresh stops.'
 	this.refreshRetries = 4;
-	
+
 	// Time zone correction in hours.
 	// You shouldn't require this unless you can't set the time zone on your computer/device properly.
 	this.timeZoneCorrection = 0;
@@ -92,7 +92,7 @@ var UISettings = (new function($)
 	// The choosen interval is saved in web-browser and then restored.
 	// The default value sets the interval on first use only.
 	this.refreshInterval = 1;
-	
+
 	// URL for communication with NZBGet via JSON-RPC
 	this.rpcUrl = './jsonrpc';
 
@@ -103,7 +103,7 @@ var UISettings = (new function($)
 	this.miniTheme = false;
 	this.showEditButtons = true;
 	this.connectionError = false;
-	
+
 	this.load = function()
 	{
 		this.refreshInterval = parseFloat(this.read('RefreshInterval', this.refreshInterval));
@@ -131,7 +131,7 @@ var UISettings = (new function($)
 		this.write('WindowTitle', this.windowTitle);
 		this.write('RefreshRetries', this.refreshRetries);
 	}
-	
+
 	this.read = function(key, def)
 	{
 		var v = localStorage.getItem(key);
@@ -156,7 +156,7 @@ var UISettings = (new function($)
 
 $(document).ready(function()
 {
-	Frontend.init(); 
+	Frontend.init();
 });
 
 
@@ -174,11 +174,12 @@ var Frontend = (new function($)
 	var switchingTheme = false;
 	var activeTab = 'Downloads';
 	var lastTab = '';
-	
+	var lastMenu = $();
+
 	this.init = function()
 	{
 		window.onerror = error;
-		
+
 		if (!checkBrowser())
 		{
 			return;
@@ -188,7 +189,7 @@ var Frontend = (new function($)
 
 		UISettings.load();
 		Refresher.init();
-		
+
 		initControls();
 		switchTheme();
 		windowResized();
@@ -206,6 +207,7 @@ var Frontend = (new function($)
 		ConfigBackupRestore.init();
 		ConfirmDialog.init();
 		UpdateDialog.init();
+		ExecScriptDialog.init();
 		AlertDialog.init();
 		ScriptListDialog.init();
 		RestoreSettingsDialog.init();
@@ -216,14 +218,15 @@ var Frontend = (new function($)
 		DownloadsMergeDialog.init();
 		DownloadsSplitDialog.init();
 		HistoryEditDialog.init();
-		
+		PurgeHistoryDialog.init();
+
 		$(window).resize(windowResized);
 
 		initialized = true;
 
-		Refresher.update();
+		authorize();
 	}
-	
+
 	function initControls()
 	{
 		mobileSafari = $.browser.safari && navigator.userAgent.toLowerCase().match(/(iphone|ipod|ipad)/) != null;
@@ -238,20 +241,12 @@ var Frontend = (new function($)
 		$('#Navbar a[data-toggle="tab"]').on('show', beforeTabShow);
 		$('#Navbar a[data-toggle="tab"]').on('shown', afterTabShow);
 		setupSearch();
-		
+
 		$('li > a:has(table)').addClass('has-table');
-
-		$(document).on("keypress", "form", function(event)
-		{
-			// since we use form-tags for a workaround for fieldset-tag,
-			// and we don't have a proper processing of ENTER-key inside forms,
-			// we disable ENTER-key to prevent automatic form submitting.
-			return event.keyCode != 13;
-		});
-
+		$(document).on('keydown', keyDown);
 		$(window).scroll(windowScrolled);
 	}
-	
+
 	function checkBrowser()
 	{
 		if ($.browser.msie && parseInt($.browser.version, 10) < 9)
@@ -263,14 +258,14 @@ var Frontend = (new function($)
 		return true;
 	}
 
-	function error(message, source, lineno) 
+	function error(message, source, lineno)
 	{
-		if (source == "")
+		if (source == '')
 		{
 			// ignore false errors without source information (sometimes happen in Safari)
 			return false;
 		}
-		
+
 		$('#FirstUpdateInfo').hide();
 		$('#ErrorAlert-title').text('Error in ' + source + ' (line ' + lineno + ')');
 		$('#ErrorAlert-text').text(message);
@@ -280,10 +275,10 @@ var Frontend = (new function($)
 		{
 			Refresher.pause();
 		}
-		
+
 		return false;
 	}
-	
+
 	this.loadCompleted = function()
 	{
 		Downloads.redraw();
@@ -298,11 +293,32 @@ var Frontend = (new function($)
 			$('#Navbar').show();
 			$('#MainTabContent').show();
 			$('#version').text(Options.option('Version'));
+			selectInitialTab();
 			windowResized();
 			firstLoad = false;
 		}
 	}
-	
+
+	function selectInitialTab()
+	{
+		var location = window.location.toString();
+		var link = null;
+		if (location.indexOf('#downloads') > -1)
+			link = 'DownloadsTabLink';
+		else if (location.indexOf('#history') > -1)
+			link = 'HistoryTabLink';
+		else if (location.indexOf('#messages') > -1)
+			link = 'MessagesTabLink';
+		else if (location.indexOf('#settings') > -1)
+			link = 'ConfigTabLink';
+		if (link)
+		{
+			$('#DownloadsTab').removeClass('fade');
+			$('#' + link).click();
+			$('#DownloadsTab').addClass('fade');
+		}
+	}
+
 	function beforeTabShow(e)
 	{
 		var tabname = $(e.target).attr('href');
@@ -316,7 +332,7 @@ var Frontend = (new function($)
 
 		lastTab = activeTab;
 		activeTab = tabname;
-		
+
 		$('#SearchBlock .search-query, #SearchBlock .search-clear').hide();
 		$('#' + activeTab + 'Table_filter, #' + activeTab + 'Table_clearfilter').show();
 
@@ -326,7 +342,7 @@ var Frontend = (new function($)
 			case 'Messages': Messages.show(); break;
 			case 'History': History.show(); break;
 		}
-		
+
 		FilterMenu.setTab(activeTab);
 	}
 
@@ -362,6 +378,56 @@ var Frontend = (new function($)
 		beforeTabShow({target: $('#DownloadsTabLink')});
 	}
 
+	function keyDown(e)
+	{
+		var key = Util.keyName(e);
+
+		var modals = $('.modal:visible');
+		if (modals.length > 0)
+		{
+			if (key === 'Enter' && !Util.wantsReturn(e.target))
+			{
+				var primaryButton = $('.btn-primary:visible', modals.last());
+				if (primaryButton.length === 1)
+				{
+					primaryButton.click();
+				}
+				return false;
+			}
+			return;
+		}
+
+		var filterBox = $('#DownloadsTable_filter, #HistoryTable_filter, #MessagesTable_filter, #ConfigTable_filter');
+		if (filterBox.is(':focus') && (key === 'Escape' || key === 'Enter'))
+		{
+			filterBox.blur();
+			return false;
+		}
+
+		if (!(Util.isInputControl(e.target) && $(e.target).is(':visible')))
+		{
+			switch (activeTab)
+			{
+				case 'Downloads': if (Downloads.processShortcut(key)) return false;
+				case 'History': if (History.processShortcut(key)) return false;
+				case 'Messages': if (Messages.processShortcut(key)) return false;
+				case 'Config': if (Config.processShortcut(key)) return false;
+			}
+			switch (key)
+			{
+				case 'Shift+D': $('#DownloadsTabLink').click(); return false;
+				case 'Shift+H': $('#HistoryTabLink').click(); return false;
+				case 'Shift+M': $('#MessagesTabLink').click(); return false;
+				case 'Shift+S': $('#ConfigTabLink').click(); return false;
+				case 'Shift+L': $('#StatusSpeed').click(); return false;
+				case 'Shift+A': $('#StatusTime').click(); return false;
+				case 'Shift+R': $('#RefreshButton').click(); return false;
+				case 'Shift+P': $('#PlayPauseButton').click(); return false;
+				case 'Shift+T': $('#ScheduledPauseButton').click(); return false;
+			}
+		}
+	}
+
 	function windowScrolled()
 	{
 		$('body').toggleClass('scrolled', $(window).scrollTop() > 0 && !UISettings.miniTheme);
@@ -396,7 +462,7 @@ var Frontend = (new function($)
 		alignPopupMenu('#StatDialog_MonthMenu', true);
 
 		alignCenterDialogs();
-		
+
 		if (initialized)
 		{
 			Downloads.resize();
@@ -442,7 +508,51 @@ var Frontend = (new function($)
 		}
 	}
 	this.alignPopupMenu = alignPopupMenu;
-	
+
+	function showPopupMenu(menu, anchor, rect)
+	{
+		var $menu = $(menu);
+		if ($menu.is(':visible'))
+		{
+			$menu.hide();
+			return;
+		}
+
+		lastMenu.hide();
+		lastMenu = $menu;
+
+		$menu.css({
+			left: rect.left + (anchor.indexOf('right') > -1 ? rect.width - $menu.outerWidth() : 0),
+			top: rect.top + (anchor.indexOf('top') > -1 ? - $menu.outerHeight() : rect.height)
+		});
+		$menu.show();
+
+		if ($menu.offset().top < $(window).scrollTop())
+		{
+			$menu.css({ top: rect.top + rect.height });
+		}
+		if ($menu.offset().left + $menu.outerWidth() > $(window).width())
+		{
+			$menu.css({ left: $(window).width() - $menu.outerWidth() });
+		}
+		if ($menu.offset().top + $menu.outerHeight() > $(window).height() + $(window).scrollTop())
+		{
+			$menu.css({ top: rect.top - $menu.outerHeight() });
+		}
+		if ($menu.offset().top < $(window).scrollTop())
+		{
+			$menu.css({ top: $(window).scrollTop() });
+		}
+
+		if (UISettings.miniTheme)
+		{
+			alignPopupMenu($menu);
+		}
+
+		$('html').on('click.PopupMenu', function () { $menu.hide(); $('html').off('click.PopupMenu'); });
+	}
+	this.showPopupMenu = showPopupMenu;
+
 	function alignCenterDialogs()
 	{
 		$.each($('.modal-center'), function(index, element) {
@@ -507,7 +617,7 @@ var Frontend = (new function($)
 			control.lastOuterWidth = control.outerWidth();
 		}
 	}
-	
+
 	function switchTheme()
 	{
 		switchingTheme = true;
@@ -557,6 +667,63 @@ var Frontend = (new function($)
 
 		switchingTheme = false;
 	}
+	
+	function authorize()
+	{
+		var formAuth = document.cookie.indexOf('Auth-Type=form') > -1;
+		if (!formAuth)
+		{
+			Refresher.update();
+			return;
+		}
+
+		function sendAuth()
+		{
+			var username = $('#LoginDialog_Username').val();
+			var password = $('#LoginDialog_Password').val();
+			var headers = [{name: 'X-Authorization', value: 'Basic ' + window.btoa(username + ':' + password)}];
+			RPC.call('version', [],
+				function(version)
+				{
+					$('#LoginDialog').modal('hide');
+					// reloading of page is needed for certain browsers to force save-password-dialog
+					document.location.reload();
+				},
+				function(err, result)
+				{
+					$('#LoginDialog_PasswordBlock').removeClass('last-group');
+					$('#LoginDialog_Error').show();
+					if (!$('#LoginDialog_Password').is(":focus"))
+					{
+						$('#LoginDialog_Username').focus();
+					}
+				},
+				0, headers);
+		}
+
+		$('#LoginDialog_Form').submit(function(e)
+			{
+				if ($('#LoginDialog_Error').is(":visible"))
+				{
+					$('#LoginDialog_Error').hide();
+					$('#LoginDialog_PasswordBlock').addClass('last-group');
+					setTimeout(sendAuth, 500);
+				}
+				else
+				{
+					setTimeout(sendAuth, 0);
+				}
+
+				return false;
+			});
+
+		// try RPC call, it may work without extra authorization
+		RPC.call('version', [], Refresher.update, function()
+			{
+				$('#LoginDialog').modal({backdrop: 'static'});
+				$('#LoginDialog_Username').focus();
+			}, 10000);
+	}
 }(jQuery));
 
 
@@ -577,7 +744,7 @@ var Refresher = (new function($)
 	var refreshing = false;
 	var refreshNeeded = false;
 	var refreshErrors = 0;
-	
+
 	this.init = function()
 	{
 		RPC.rpcUrl = UISettings.rpcUrl;
@@ -647,7 +814,7 @@ var Refresher = (new function($)
 				return;
 			}
 		}
-		
+
 		Refresher.pause();
 		UISettings.connectionError = true;
 		$('#FirstUpdateInfo').hide();
@@ -659,7 +826,7 @@ var Refresher = (new function($)
 			// stop animations
 			Status.redraw();
 		}
-		
+
 		$('html, body').animate({scrollTop: 0 }, 400);
 	};
 
@@ -680,17 +847,25 @@ var Refresher = (new function($)
 		scheduleNextRefresh();
 	}
 
+	this.isPaused = function()
+	{
+		return refreshPaused > 0;
+	}
+
 	this.pause = function()
 	{
 		clearTimeout(refreshTimer);
 		refreshPaused++;
 	}
 
-	this.resume = function()
+	this.resume = function(wantUpdate)
 	{
 		refreshPaused--;
-
-		if (refreshPaused === 0 && UISettings.refreshInterval > 0)
+		if (refreshPaused === 0 && wantUpdate)
+		{
+			this.update();
+		}
+		else if (refreshPaused === 0 && UISettings.refreshInterval > 0)
 		{
 			countSeconds();
 		}
@@ -705,7 +880,7 @@ var Refresher = (new function($)
 			scheduleNextRefresh();
 		}
 	}
-	
+
 	function refreshClick()
 	{
 		if (indicatorFrame > 10)
@@ -824,10 +999,11 @@ var ConfirmDialog = (new function($)
 
 	// Controls
 	var $ConfirmDialog;
-	
+
 	// State
 	var actionCallback;
-	
+	var confirmed = false;
+
 	this.init = function()
 	{
 		$ConfirmDialog = $('#ConfirmDialog');
@@ -856,10 +1032,11 @@ var ConfirmDialog = (new function($)
 		{
 			initCallback($ConfirmDialog);
 		}
-		
+
 		Util.centerDialog($ConfirmDialog, true);
+		confirmed = false;
 		$ConfirmDialog.modal({backdrop: 'static'});
-		
+
 		// avoid showing multiple backdrops when the modal is shown from other modal
 		var backdrops = $('.modal-backdrop');
 		if (backdrops.length > 1)
@@ -870,6 +1047,11 @@ var ConfirmDialog = (new function($)
 
 	function hidden()
 	{
+		if (confirmed)
+		{
+			actionCallback($ConfirmDialog);
+		}
+
 		// confirm dialog copies data from other nodes
 		// the copied DOM nodes must be destroyed
 		$('#ConfirmDialog_Title').empty();
@@ -880,7 +1062,7 @@ var ConfirmDialog = (new function($)
 	function click(event)
 	{
 		event.preventDefault(); // avoid scrolling
-		actionCallback($ConfirmDialog);
+		confirmed = true;
 		$ConfirmDialog.modal('hide');
 	}
 }(jQuery));
@@ -894,7 +1076,7 @@ var AlertDialog = (new function($)
 
 	// Controls
 	var $AlertDialog;
-	
+
 	this.init = function()
 	{
 		$AlertDialog = $('#AlertDialog');
@@ -915,7 +1097,7 @@ var AlertDialog = (new function($)
 var PopupNotification = (new function($)
 {
 	'use strict';
-	
+
 	this.show = function(alert, completeFunc)
 	{
 		if (UISettings.showNotifications || $(alert).hasClass('alert-error'))
